import os
import time
import unittest
from unittest import SkipTest
import filecmp
from filecmp import dircmp
import MAST
from MAST.ingredients.pmgextend.structure_extensions import StructureExtensions
import shutil
import pymatgen
import numpy as np
from MAST.utility import dirutil
testname ="structure_extensions_test"
testdir = dirutil.get_test_dir(testname)
#oldcontrol = os.getenv("MAST_CONTROL")
#oldrecipe = os.getenv("MAST_RECIPE_PATH")
#oldscratch = os.getenv("MAST_SCRATCH")
#print "Old directories:"
#print oldcontrol
#print oldrecipe
#print oldscratch


class TestSE(unittest.TestCase):
    """Test StructureExtensions
    """
    def setUp(self):
        os.chdir(testdir)

    def tearDown(self):
        pass
    def test_induce_defect_frac(self):
        perfect = pymatgen.io.vaspio.Poscar.from_file("POSCAR_perfect").structure
        compare_vac1 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_vac1").structure
        compare_int1 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_int1").structure
        compare_sub1 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_sub1").structure
        coord_type='fractional'
        threshold=1.e-4
        vac1={'symbol':'O', 'type': 'vacancy', 'coordinates':  np.array([0.25, 0.75, 0.25])}
        int1={'symbol':'Ni', 'type': 'interstitial', 'coordinates': np.array([0.3, 0.3, 0.3])}
        sub1={'symbol':'Fe', 'type': 'substitution','coordinates':np.array([0.25, 0.25,0.75])}
        sxtend = StructureExtensions(struc_work1=perfect)
        struc_vac1 = sxtend.induce_defect(vac1, coord_type, threshold)
        struc_int1 = sxtend.induce_defect(int1, coord_type, threshold)
        struc_sub1 = sxtend.induce_defect(sub1, coord_type, threshold)
        self.assertEqual(struc_vac1,compare_vac1)
        self.assertEqual(struc_int1,compare_int1)
        self.assertEqual(struc_sub1,compare_sub1)
        self.assertEqual(struc_vac1.lattice,compare_vac1.lattice)
        self.assertEqual(struc_int1.lattice,compare_int1.lattice)
        self.assertEqual(struc_sub1.lattice,compare_sub1.lattice)
        self.assertEqual(struc_vac1.sites,compare_vac1.sites)
        self.assertEqual(struc_int1.sites,compare_int1.sites)
        self.assertEqual(struc_sub1.sites,compare_sub1.sites)
    
    def test_sort_structure_and_neb_lines(self):
        perfect1 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_defectgroup1").structure
        compare_sorted1 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_sorted1").structure
        perfect2 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_defectgroup2").structure
        compare_sorted2 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_sorted2").structure
        neblines = list()
        neblines.append(["Cr","0.3 0 0","0 0 0"])
        neblines.append(["Ni","0.6 0 0","0.3 0 0"])
        sxtend1 = StructureExtensions(struc_work1=perfect1, name=testdir)
        sorted1 = sxtend1.sort_structure_and_neb_lines(neblines,"00",3)
        sxtend2 = StructureExtensions(struc_work1=perfect2, name=testdir)
        sorted2 = sxtend2.sort_structure_and_neb_lines(neblines,"04",3)
        self.assertEqual(sorted1, compare_sorted1)
        self.assertEqual(sorted2, compare_sorted2)
        self.assertEqual(sorted1.lattice, compare_sorted1.lattice)
        self.assertEqual(sorted2.lattice, compare_sorted2.lattice)
        self.assertEqual(sorted1.sites, compare_sorted1.sites)
        self.assertEqual(sorted2.sites, compare_sorted2.sites)
        neblines = list()
        neblines.append(["Cr","0.29 0.05 0.05","0.01 0.01 0.98"])
        neblines.append(["Ni","0.61 0.99 0.98","0.25 0.01 0.97"])
        sxtend1 = StructureExtensions(struc_work1=perfect1, name=testdir)
        sorted1 = sxtend1.sort_structure_and_neb_lines(neblines,"00",3)
        sxtend2 = StructureExtensions(struc_work1=perfect2, name=testdir)
        sorted2 = sxtend2.sort_structure_and_neb_lines(neblines,"04",3)
        self.assertEqual(sorted1, compare_sorted1)
        self.assertEqual(sorted2, compare_sorted2)
        self.assertEqual(sorted1.lattice, compare_sorted1.lattice)
        self.assertEqual(sorted2.lattice, compare_sorted2.lattice)
        self.assertEqual(sorted1.sites, compare_sorted1.sites)
        self.assertEqual(sorted2.sites, compare_sorted2.sites)
        neblines = list()
        neblines.append(["Cr","0.0 0.9 0.8","0.0 0.8 0.7"])
        neblines.append(["Cr","0.4 0.2 0.1","0.3 0.3 0.2"])
        neblines.append(["Cr","0.29 0.05 0.05","0.01 0.01 0.98"])
        neblines.append(["Ni","0.61 0.99 0.98","0.25 0.01 0.97"])
        perfect3 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_defectgroup3").structure
        #print perfect3.get_sorted_structure()
        perfect4 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_defectgroup4").structure
        #print perfect4.get_sorted_structure()
        sxtend3 = StructureExtensions(struc_work1=perfect3, name=testdir)
        sorted3 = sxtend3.sort_structure_and_neb_lines(neblines,"00",3)
        #print sorted3
        sxtend4 = StructureExtensions(struc_work1=perfect4, name=testdir)
        sorted4 = sxtend4.sort_structure_and_neb_lines(neblines,"04",3)
        #print sorted4
        compare_sorted3 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_sorted3").structure
        #print compare_sorted3
        compare_sorted4 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_sorted4").structure
        #print compare_sorted4
        self.assertEqual(sorted3, compare_sorted3)
        self.assertEqual(sorted4, compare_sorted4)
        self.assertEqual(sorted3.lattice, compare_sorted3.lattice)
        self.assertEqual(sorted4.lattice, compare_sorted4.lattice)
        self.assertEqual(sorted3.sites, compare_sorted3.sites)
        self.assertEqual(sorted4.sites, compare_sorted4.sites)

    def test_sort_structure_and_neb_lines_really_scrambled(self):
        raise SkipTest
        neblines = list()
        neblines.append(["Cr","0.0 0.9 0.8","0.0 0.8 0.7"])
        neblines.append(["Cr","0.4 0.2 0.1","0.3 0.3 0.2"])
        neblines.append(["Cr","0.29 0.05 0.05","0.01 0.01 0.98"])
        neblines.append(["Ni","0.61 0.99 0.98","0.25 0.01 0.97"])
        perfect3 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_defectgroup3_scrambled").structure
        #print perfect3.get_sorted_structure()
        #print perfect4.get_sorted_structure()
        sxtend3 = StructureExtensions(struc_work1=perfect3)
        sorted3 = sxtend3.sort_structure_and_neb_lines(neblines,"00",3)
        compare_sorted3 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_sorted3").structure
        self.assertEqual(sorted3, compare_sorted3)
        self.assertEqual(sorted3.lattice, compare_sorted3.lattice)
        self.assertEqual(sorted3.sites, compare_sorted3.sites)
        perfect3 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_defectgroup3_really_scrambled").structure
        #print perfect3.get_sorted_structure()
        #print perfect4.get_sorted_structure()
        sxtend3 = StructureExtensions(struc_work1=perfect3)
        sorted3 = sxtend3.sort_structure_and_neb_lines(neblines,"00",3)
        compare_sorted3 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_sorted3").structure
        self.assertEqual(sorted3, compare_sorted3)
        self.assertEqual(sorted3.lattice, compare_sorted3.lattice)
        self.assertEqual(sorted3.sites, compare_sorted3.sites)


    def test_interpolation(self):
        ep1 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_ep1").structure
        ep2 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_ep2").structure
        compare_im1 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_im1").structure
        compare_im2 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_im2").structure
        compare_im3 = pymatgen.io.vaspio.Poscar.from_file("POSCAR_im3").structure
        sxtend = StructureExtensions(struc_work1 = ep1, struc_work2 = ep2)
        slist = sxtend.do_interpolation(3)
        self.assertEqual(slist[0],ep1)
        self.assertEqual(slist[1],compare_im1)
        self.assertEqual(slist[2],compare_im2)
        self.assertEqual(slist[3],compare_im3)
        self.assertEqual(slist[4],ep2)
    def test_get_sd_array(self):
        perfect = pymatgen.io.vaspio.Poscar.from_file("POSCAR_perfect").structure
        sxtend = StructureExtensions(struc_work1=perfect, name=testdir)
        mysd = sxtend.get_sd_array("0.5 0.5 0.5", 3)
        #print mysd
        myarr=np.zeros([40,3],bool)
        for idx in [8,20,25,26,28,30,32,33,36,37,38,39,40]:
            myarr[idx-1][0]=True
            myarr[idx-1][1]=True
            myarr[idx-1][2]=True
        self.assertEqual(sum(np.fabs(sum(mysd-myarr))),0)
    def test_get_sd_array_periodic_boundary(self):
        perfect = pymatgen.io.vaspio.Poscar.from_file("POSCAR_perfect").structure
        sxtend = StructureExtensions(struc_work1=perfect, name=testdir)
        mysd = sxtend.get_sd_array("0.95 0.95 0.95", 1, 0.07)
        #print mysd
        myarr=np.zeros([40,3],bool)
        for idx in [1]:
            myarr[idx-1][0]=True
            myarr[idx-1][1]=True
            myarr[idx-1][2]=True
        self.assertEqual(sum(np.fabs(sum(mysd-myarr))),0)
    def test_multiple_sd_array(self):
        perfect = pymatgen.io.vaspio.Poscar.from_file("POSCAR_perfect").structure
        sxtend = StructureExtensions(struc_work1=perfect, name=testdir)
        mysdlist = sxtend.get_multiple_sd_array("0.5 0.5 0.5", 1)
        mylist=list()
        mylist.append(np.zeros([40,3],bool))
        mylist.append(np.zeros([40,3],bool))
        mylist.append(np.zeros([40,3],bool))
        mylist[0][8-1][0]=True
        mylist[1][8-1][1]=True
        mylist[2][8-1][2]=True
        self.assertEqual(sum(np.fabs(sum(mysdlist[0]-mylist[0]))),0)
        self.assertEqual(sum(np.fabs(sum(mysdlist[1]-mylist[1]))),0)
        self.assertEqual(sum(np.fabs(sum(mysdlist[2]-mylist[2]))),0)
    def test_graft_coordinates(self):
        perfect = pymatgen.io.vaspio.Poscar.from_file("POSCAR_perfect").structure
        coordsonly = pymatgen.io.vaspio.Poscar.from_file("POSCAR_coordinates").structure
        compare_grafted = pymatgen.io.vaspio.Poscar.from_file("POSCAR_grafted").structure
        sxtend = StructureExtensions(struc_work1=perfect)
        grafted = sxtend.graft_coordinates_onto_structure(coordsonly)
        self.assertEqual(grafted, compare_grafted)
        self.assertEqual(grafted.lattice, compare_grafted.lattice)
        self.assertEqual(grafted.sites, compare_grafted.sites)

    def test_strain_lattice(self):
        perfect = pymatgen.io.vaspio.Poscar.from_file("POSCAR_unstrained").structure
        sxtend = StructureExtensions(struc_work1=perfect)
        strained = sxtend.strain_lattice(" 0.98 0.92 1.03  \n")
        strain_compare = pymatgen.io.vaspio.Poscar.from_file("POSCAR_strained").structure
        self.assertEqual(strained, strain_compare)
        self.assertEqual(strained.lattice, strain_compare.lattice)
        self.assertEqual(strained.sites, strain_compare.sites)
    def test_scale_structure(self):
        hcp = pymatgen.io.vaspio.Poscar.from_file("POSCAR_HCP").structure
        sxtend = StructureExtensions(struc_work1=hcp, scaling_size="2 2 2", name=testdir)
        scaled = sxtend.scale_structure()
        self.assertEqual(scaled, pymatgen.io.vaspio.Poscar.from_file("POSCAR_HCP_222").structure)
        self.assertEqual(scaled.lattice, pymatgen.io.vaspio.Poscar.from_file("POSCAR_HCP_222").structure.lattice)
        self.assertEqual(scaled.sites.sort(), pymatgen.io.vaspio.Poscar.from_file("POSCAR_HCP_222").structure.sites.sort())
    def test_scale_defect(self):
        perfect = pymatgen.io.vaspio.Poscar.from_file("POSCAR_perfect").structure
        scalingsize = "2 2 2"
        sxtend = StructureExtensions(struc_work1=perfect, scaling_size=scalingsize, name=testdir)
        scaled = sxtend.scale_structure()
        sxtend2 = StructureExtensions(struc_work1=scaled, struc_work2=perfect, scaling_size=scalingsize, name=testdir)
        vac1={'symbol':'O', 'type': 'vacancy', 'coordinates':  np.array([0.25, 0.75, 0.25])}
        defected =  sxtend2.scale_defect(vac1,'fractional',0.0001)
        int1={'symbol':'Ni', 'type': 'interstitial', 'coordinates': np.array([0.3, 0.3, 0.3])}
        sxtend3 = StructureExtensions(struc_work1=defected, struc_work2=perfect, scaling_size=scalingsize, name=testdir)
        defected2 = sxtend3.scale_defect(int1,'fractional',0.0001)
        sub1={'symbol':'Fe', 'type': 'substitution','coordinates':np.array([0.25, 0.25,0.75])}
        sxtend4 = StructureExtensions(struc_work1=defected2, struc_work2=perfect, scaling_size=scalingsize, name=testdir)
        defected3 = sxtend4.scale_defect(sub1,'fractional',0.0001)
        self.assertEqual(pymatgen.io.vaspio.Poscar.from_file("POSCAR_scaled_defected").structure, defected3)



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Glen Jenness
# Last updated: 2013-07-01
##############################################################
import sys, os

import pymatgen as pmg
from pymatgen.io.vaspio.vasp_output import Vasprun, Outcar
from pymatgen.io.smartio import read_structure
from MAST.utility import MASTError
from MAST.utility import MASTFile
#from MAST.utility import PickleManager
from MAST.utility.defect_formation_energy.potential_alignment import PotentialAlignment
from MAST.utility.defect_formation_energy.gapplot import GapPlot
from MAST.utility import Metadata
from MAST.parsers import InputParser

class DefectFormationEnergy:
    """Class for calculating the defect formation energy for a completed MAST
        run.
    """

    def __init__(self, directory=None, plot_threshold=0.01):
        self.directory = directory
        self.plot_threshold = plot_threshold
        ipparser = InputParser(inputfile=os.path.join(self.directory,'input.inp'))
        self.input_options = ipparser.parse()
        #pm = PickleManager(self.directory + '/input_options.pickle')
        #self.input_options = pm.load_variable()
        from MAST.recipe.recipesetup import RecipeSetup
        self.recipe_setup = RecipeSetup(inputOptions=self.input_options,recipeFile=os.path.join(self.directory,'personal_recipe.txt'),workingDirectory=self.directory)
        self.recipe_plan = self.recipe_setup.start()
        #PickleManager(self.directory + '/mast.pickle').load_variable()

        self.final_ingredients = list()
        ingredientlist = self.recipe_plan.ingredients.keys()
        updatekeys = self.recipe_plan.update_methods.keys()
        for ingredient in ingredientlist:
            if not (ingredient in updatekeys): # has no children
                self.final_ingredients.append(ingredient)
            else:
                if len(self.recipe_plan.update_methods[ingredient]) == 0:
                    self.final_ingredients.append(ingredient)
        #print self.final_ingredients

        self.e_defects = dict()
    def _calculate_defect_formation_energies(self):
        try:
            perf_dir = [ingredient for ingredient in self.final_ingredients if ('perfect' in ingredient)][0]
        except IndexError:
            raise MASTError(self.__class__.__name__, "A perfect final directory (has no children and has the word 'perfect' in its name) could not be found. Check recipe %s for a perfect directory." % self.directory)

        def_dir = [ingredient for ingredient in self.final_ingredients if 'perfect' not in ingredient] 
        #for whatever in sorted(def_dir):
            #print whatever

        defects = self.input_options.get_item('defects', 'defects')
        chempot = self.input_options.get_item('chemical_potentials')

        perf_meta = Metadata(metafile='%s/%s/metadata.txt' % (self.directory, perf_dir))
        e_perf = float(perf_meta.read_data('energy'))
        efermi = self.get_fermi_energy(perf_dir)
        struct_perf = self.get_structure(perf_dir)

        perf_species = dict()
        for site in struct_perf:
            if (str(site.specie) not in perf_species):
                perf_species[str(site.specie)] = 1
            else:
                perf_species[str(site.specie)] += 1

        #print 'Perfect composition: ', perf_species

        # First loop through the conditions for the defects
        for conditions, potentials in chempot.items():
            print 'Conditions:  %s' % conditions.upper()
            self.e_defects[conditions] = dict()

            # Loop through each defect
            for ddir in sorted(def_dir):
                def_meta = Metadata(metafile='%s/%s/metadata.txt' % (self.directory, ddir))
                #print def_meta
                label = def_meta.read_data('defect_label')
                charge = int(def_meta.read_data('charge'))
                energy = float(def_meta.read_data('energy'))
                structure = self.get_structure(ddir)

                if (label not in self.e_defects[conditions]):
                    self.e_defects[conditions][label] = list()

                print 'Calculating DFEs for defect %s with charge %3i.' % (label, charge)

                # Find out how many atoms of each type are in the defects
                def_species = dict()
                for site in structure.sites:
                    if (site.specie not in def_species):
                        def_species[site.specie] = 1
                    else:
                        def_species[site.specie] += 1

                # Find the differences in the number of each atom type
                # between the perfect and the defect
                struct_diff = dict()
                for specie, number in def_species.items():
                    try:
                        nperf = perf_species[str(specie)]
                    except KeyError:
                        nperf = 0
                    struct_diff[str(specie)] = number - nperf

                # Get the potential alignment correction
                alignment = self.get_potential_alignment(perf_dir, ddir)

                # Calculate the base DFE energy
                e_def = energy - e_perf # E_defect - E_perf
                for specie, number in struct_diff.items():
                    mu = potentials[str(specie)]
                    #print str(specie), mu, number
                    e_def -= (number * mu)
                e_def += charge * (efermi + alignment) # Add in the shift here!
                #print '%-15s%-5i%12.5f%12.5f%12.5f%12.5f' % (label.split('_')[1], charge, energy, e_perf, efermi, alignment)
                print 'DFE = %f' % e_def
                self.e_defects[conditions][label].append( (charge, e_def) )

        #print e_defects
        #return e_defects

    def get_total_energy(self, directory):
        """Returns the total energy from a directory"""

        abspath = '%s/%s/' % (self.directory, directory)

        if ('vasprun.xml' in os.listdir(abspath)):
        # Modified from the PyMatGen Vasprun.final_energy() function to return E_0
            return Vasprun('%s/vasprun.xml' % abspath).ionic_steps[-1]["electronic_steps"][-1]["e_0_energy"]

    def get_fermi_energy(self, directory):
        """Returns the Fermi energy from a directory"""
        abspath = '%s/%s/' % (self.directory, directory)

        if ('vasprun.xml' in os.listdir(abspath)):
            return Vasprun('%s/vasprun.xml' % abspath).efermi
        elif ('OUTCAR' in os.listdir(abspath)):
            return Outcar('%s/OUTCAR' % abspath).efermi

    def get_structure(self, directory):
        """Returns the final structure from an optimization"""

        abspath = '%s/%s/' % (self.directory, directory)

        if ('vasprun.xml' in os.listdir(abspath)):
            return Vasprun('%s/vasprun.xml' % abspath).final_structure
        elif ('CONTCAR' in os.listdir(abspath)):
            return pmg.read_structure('%s/CONTCAR' % abspath)

    def get_potential_alignment(self, perf_dir, def_dir):
        """Returns the potential alignment correction used in charge defects"""

        abs_path_perf = '%s/%s/' % (self.directory, perf_dir)
        abs_path_def = '%s/%s/' % (self.directory, def_dir)

        pa = PotentialAlignment()

        if ('OUTCAR' in os.listdir(abs_path_perf)):
            perfect_info = pa.read_outcar('%s/%s' % (abs_path_perf, 'OUTCAR'))
            defect_info = pa.read_outcar('%s/%s' % (abs_path_def, 'OUTCAR'))

            return pa.get_potential_alignment(perfect_info, defect_info)

    def get_defect_formation_energies(self):
        if not self.e_defects:
            self._calculate_defect_formation_energies()

        return self.e_defects

    @property
    def defect_formation_energies(self):
        return self.get_defect_formation_energies()

    @property
    def dfe(self):
        return self.get_defect_formation_energies()

    def print_table(self):
        if not self.e_defects:
            self._calculate_defect_formation_energies()

        myfile = MASTFile()
        for conditions, defects in self.e_defects.items():
            myfile.data.append('\n\nDefect formation energies for %s conditions.\n' % conditions.upper())
            myfile.data.append('%-20s | %10s\n' % ('Defect', 'DFE'))
            myfile.data.append('---------------------------------\n')
            for defect, energies in defects.items():
                for energy in energies:
                    myfile.data.append('%-14s(q=%2i) | %8.4f\n' % (defect, energy[0], energy[1]))
                myfile.data.append(str()) # Add a blank line here
            myfile.data.append('---------------------------------\n')
        myfile.to_file(os.path.join(os.getcwd(),"dfe.txt"))
        for line in myfile.data:
            print line.strip()




from ase import Atom, Atoms
from ase.io import read, write
import math
import numpy

def finite_size_scale(standard, ssize, primordial, fsize, psize=[1,1,1],writefile=True):
    """Function to perform finite size scaling for defect structure relaxation
    **Assumes cell information can be retrieved by reading file**
    Inputs:
        standard = File of structure containing defect
        ssize = Supercell size of structure with defect (list of size 3)
        primordial = File of structure for basic unit of perfect cell
        psize = Supercell size of structure for padding. Default = [1,1,1] (list of size 3)
        fsize = Desired supercell size of final structure (list of size 3)
    Outputs:
        POSCAR file of structure containing defect and padding"""
    
    # Check if the input sizes work out with the desired final size
    padding = [0,0,0]
    srcon = [0,0,0]
    for i in range(3):
        diff = fsize[i] - ssize[i]
        if diff < 0:
            raise RuntimeError('Desired final size of the structure must be larger than existing defect structure size. Defect Size = '+repr(ssize)+' Final Size = '+repr(fsize))
        elif diff >= 0:
             if math.fmod(diff,psize[i]):
                raise RuntimeError('Primordial structure and defect structure sizes cannot be used to form desired final size.  Reduce size of primordial structure. Defect Size = '+repr(ssize)+' Final Size = '+repr(fsize)+' Primordial size = '+repr(psize))
             else:
                padding[i] = diff/psize[i]
    
    # Load the defect structure and primordial structure
    defst = read(standard)
    pst = read(primordial)
    
    # Pad the structure
    positions = pst.get_positions()
    syms = pst.get_chemical_symbols()
    final = defst.copy()
    lv = [one/ssize for one in defst.get_cell()]
    vect = []
    for m0 in range(padding[0]):
        for m1 in numpy.arange(0,fsize[1],psize[1]):
            for m2 in numpy.arange(0,fsize[2],psize[2]):
                vect.append([ssize[0]+m0*psize[0],m1,m2])

    for m1 in range(padding[1]):
        for m0 in numpy.arange(0,ssize[0],psize[0]):
            for m2 in numpy.arange(0,fsize[2],psize[2]):
                vect.append([m0,ssize[1]+m1*psize[1],m2])

    for m2 in range(padding[2]):
        for m0 in numpy.arange(0,ssize[0],psize[0]):
            for m1 in numpy.arange(0,ssize[1],psize[1]):
                vect.append([m0,m1,ssize[2]+m2*psize[2]])

    for m0,m1,m2 in vect:
        npos = positions + numpy.dot((m0, m1, m2), lv)
        for i in range(len(npos)):
            final.append(Atom(symbol=syms[i],position=npos[i]))
    
    final.set_cell(numpy.array([fsize[c] * lv[c] for c in range(3)]))
    
    # Write output as POSCAR
    if writefile:
        write('POSCAR_Final', final)
    
    return final

# Potential issues:
#    --Doesn't account for internal periodicity of relaxed defect structure.  
#        -Could add this by checking distance
#    --Currently uses ASE but should be able to switch to pymatgen without too much issue
#    --Assumes defect structure is aligned from [0,0,0]

if __name__ == "__main__":
    #Sanity checks    
    fil=open('Test_FSS.xyz','a')
    
    #Test 1 - Final size cannot be constructed directly from primordial structure
    from ase.lattice.cubic import BodyCenteredCubic as BCC
    ssize=[3,3,3]
    fsize=[7,3,3]
    psize=[2,3,3]
    defectstructure = BCC('Fe', size = ssize)
    primordialstructure = BCC('Fe', size = psize)
    f = BCC('Fe', size = fsize)
    write('POSCAR_defect1',defectstructure)
    write('POSCAR_prim1',primordialstructure)
    final = finite_size_scale('POSCAR_defect1', ssize, 'POSCAR_prim1', fsize, psize)
    write(fil,f,'xyz')
    write(fil,final,'xyz')
    fil.flush()
    
    #Test 2 - Test in all 3 dimensions
    ssize=[2,2,2]
    fsize=[4,4,4]
    psize=[1,1,1]
    defectstructure = BCC('Fe', size = ssize)
    primordialstructure = BCC('Fe', size = psize)
    f = BCC('Fe', size = fsize)
    write('POSCAR_defect2',defectstructure)
    write('POSCAR_prim2',primordialstructure)
    final = finite_size_scale('POSCAR_defect2', ssize, 'POSCAR_prim2', fsize, psize)
    write(fil,f,'xyz')
    write(fil,final,'xyz')
    fil.flush()
    
    #Test 3 - Test for not cubic lattice
    from ase.lattice.spacegroup import crystal
    a = 3.21
    c = 5.21
    mg = crystal('Mg', [(1./3., 2./3., 3./4.)], spacegroup=194,cellpar=[a, a, c, 90, 90, 120])
    defectstructure = mg.repeat(ssize)
    primordialstructure = mg.repeat(psize)
    f = mg.repeat(fsize)
    write('POSCAR_defect3',defectstructure)
    write('POSCAR_prim3',primordialstructure)
    final = finite_size_scale('POSCAR_defect3', ssize, 'POSCAR_prim3', fsize, psize)
    write(fil,f,'xyz')
    write(fil,final,'xyz')
    fil.flush()
    fil.close()
    
    


############
NTYPES
DISP
DOSIN
DOSEND
DOSSMEAR
DOSSTEP
DXSTART
EIGSIZE
IPRINT
LCENTRAL
LDRIFT
LEIGEN
LFORCEOUT
LFREE
LGAMMA
LNOSYM
LRECIP
LSUPER
LSYMM
MASS
NAME
NCYCLESEIG
ND
NDIM
NTI
PTEMP
QA
QB
QC
QI
QF
SYMPREC
TEMPERATURE
USETHIS
############



import random
from MAST.structopt.tools import get_best

def random_pick(pop, nkeep, Optimizer):
	"""Selection function that randomly choose structures from a population to survive.
	"""
	pop = get_best(pop,len(pop))
	newpop = []
	for i in range(nkeep):
		sel = random.choice(pop)
		newpop.append(sel)
	return newpop


import matplotlib.pyplot as plt

def read_genealogy(filename):
	"""Function to read genealogy and generate individual tree
	NEEDS DEVELOPMENT
	"""
	f = open(filename,'r')
	line0=f.readline()
	indhist=[[value] for value in line0.split()]
	for line in f.readlines():
		i=0
		for value in line.split():
			indhist[i].append(value)
			i+=1
	f.close()
	numinds=len(indhist)
	gens=len(indhist[0])
	xs=[1000/numinds*(n+1) for n in range(numinds)]
	ys=[1000-1000.0/float(gens)*(n+1) for n in range(gens)]
	fig = plt.figure()
	#figsize=10 #max(numinds,gens)
	fig.set_size_inches(numinds, gens, forward=True)
	ax1=fig.add_subplot(111)
	ax1.set_frame_on(False)
	ax1.set_xticks([])
	ax1.set_yticks([])
	ax1.axis('off')
	for i in range(numinds):
		prev=None
		for j in range(gens):
			if indhist[i][j] !=prev:
				ax1.text(xs[i],ys[j],indhist[i][j])
				prev=indhist[i][j]
	#plt.xlim([0,1000])
	#plt.ylim([0,1000])
	lines=[]
	for i in range(numinds):
		for j in range(gens):
			if ')' in indhist[i][j]:
				a=indhist[i][j].split('(')[1].split(')')
				ind=[int(one) for one in a[0].split('+')]
				if 'm' in indhist[i][j]:
					lines.append([xs[ind[0]],ys[j-1],xs[i],ys[j],'b'])
					lines.append([xs[ind[1]],ys[j-1],xs[i],ys[j],'b'])
				else:
					lines.append([xs[ind[0]],ys[j-1],xs[i],ys[j],'k'])
					lines.append([xs[ind[1]],ys[j-1],xs[i],ys[j],'k'])
			elif 'm' in indhist[i][j]:
				a=indhist[i][j].split('m')
				ind=int(a[0])
				lines.append([xs[ind],ys[j-1],xs[i],ys[j],'b'])

	for one in lines:
		ax1.plot([one[0], one[2]], [one[1], one[3]], color=one[4])

	plt.savefig('geneologytree-'+filename+'.png')




from MAST.structopt.generate import gen_pop_box, gen_pop_sphere
from MAST.structopt.generate.Individual import Individual

def get_crystal_indiv(Optimizer):
    """
    Function to generate an structopt Individual class object containing a crystal structure.
    Inputs:
    	Optimizer = structopt Optimizer class
    Outputs:
    	individ = structopt Individual class object containing crystal structure data
    """
    if 'sphere' in Optimizer.generate_flag:
        outts = gen_pop_sphere(Optimizer.atomlist,Optimizer.size,Optimizer.cell_shape_options)
    else:
        outts = gen_pop_box(Optimizer.atomlist,Optimizer.size,Optimizer.cell_shape_options)
    ind = outts[0]
    Optimizer.output.write(outts[1])
    individ = Individual(ind)
    return individ



import random
import math
import numpy

def scale_size(indiv, Optimizer):
    """Move function to change the cell size for crystals or scales size of a cluster
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    sgn=random.randint(0,1)
    if Optimizer.structure=='Crystal':
        ocell=indiv[0].get_cell()
        cell=ocell
        scale=(sgn+random.random())*ocell
    else:
        ocell=indiv[0].get_cell()
        cell_max=numpy.maximum.reduce(indiv[0].get_positions())
        cell_min=numpy.minimum.reduce(indiv[0].get_positions())
        cell=cell_max-cell_min
        for l in range(len(cell)): 
            cell[l]=math.ceil(cell[l])
            if cell[l]<=1:
                cell[l]=1.5
        indiv[0].set_cell(cell)
        cell=indiv[0].get_cell()
        scale=(sgn+random.random())*cell
    xlen=scale[0][0]-scale[0][1]*scale[1][0]/scale[1][1]
    ylen=scale[1][1]-scale[1][0]*scale[0][1]/scale[0][0]
    zlen=scale[2][2]-scale[2][1]*scale[1][2]/scale[1][1]
    while xlen < 2 or ylen < 2 or zlen < 2:
        sgn=random.randint(0,1)
        scale=(sgn+random.random())*cell
        xlen=scale[0][0]-scale[0][1]*scale[1][0]/scale[1][1]
        ylen=scale[1][1]-scale[1][0]*scale[0][1]/scale[0][0]
        zlen=scale[2][2]-scale[2][1]*scale[1][2]/scale[1][1]
    indiv[0].set_cell(scale, scale_atoms=True)
    if Optimizer.structure != 'Crystal':
        indiv[0].set_cell(ocell)
    Optimizer.output.write('Scaling Mutation performed on individual\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    Optimizer.output.write('Factor = '+repr(scale)+'\n')
    Optimizer.output.write(repr(indiv[0])+'\n')
    muttype='SSz'+repr(sgn)
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


from pymatgen.io.vaspio import Poscar
from pymatgen.io.vaspio import Outcar
from pymatgen.io.vaspio import Potcar
from pymatgen.io.vaspio import Incar
from pymatgen.io.vaspio import Kpoints
from pymatgen.core.structure import Structure
from MAST.utility import dirutil
from MAST.utility.mastfile import MASTFile
from MAST.utility import MASTError
from MAST.ingredients.checker import BaseChecker
from MAST.ingredients.checker import VaspChecker
import os
import logging
import pymatgen
import numpy as np
import time
class PhonChecker(BaseChecker):
    """PHON checker functions
    """
    def __init__(self, **kwargs):
        allowed_keys = {
            'name' : (str, str(), 'Name of directory'),
            'program_keys': (dict, dict(), 'Dictionary of program keywords'),
            'structure': (Structure, None, 'Pymatgen Structure object')
            }
        BaseChecker.__init__(self, allowed_keys, **kwargs)

    def is_complete(self):
        """Check if PHON thermo run is complete."""
        if os.path.isfile(os.path.join(self.keywords['name'], "THERMO")):
            return True
        elif os.path.isfile(os.path.join(self.keywords['name'],"FREQ")):
            return True
        else:
            return False

    def is_ready_to_run(self):
        """Check if PHON is ready to run."""
        dirname = self.keywords['name']
        notready=0
        if not(os.path.isfile(dirname + "/FORCES")):
            notready = notready + 1
        if not(os.path.isfile(dirname + "/POSCAR")):
            notready = notready + 1
        if not(os.path.isfile(dirname + "/INPHON")):
            notready = notready + 1
        if not(os.path.isfile(dirname + "/submit.sh")):
            notready = notready + 1
        if notready > 0:
            return False
        else:
            return True

    def _phon_poscar_setup(self):
        """Set up a PHON POSCAR file. Strip out the "elements" line (that is,
            use VASP version 4 format. Also strip out anything beneath the atoms
            line.
        """
        name = self.keywords['name']
        pospath = os.path.join(name, "POSCAR")
        prepath = os.path.join(name, "POSCAR_prePHON")
        if os.path.isfile(pospath): #Already done. Return.
            return
        my_poscar = Poscar.from_file(prepath) 
        my_poscar.selective_dynamics=None #unset SD if it is set
        my_poscar.velocities=None #unset velocities
        dirutil.lock_directory(name)
        my_poscar.write_file(pospath)
        dirutil.unlock_directory(name)
        #pick up a copy and strip out the elements line.
        mypfile = MASTFile(pospath)
        myline6=mypfile.get_line_number(6)
        if myline6.strip().split()[0].isalpha:
            mypfile.modify_file_by_line_number(6,"D")
        mypfile.to_file(pospath)
        return



    def _phon_inphon_get_non_mast_keywords(self):
        """Sort out the non-PHON keywords and make a dictionary."""
        inphon_dict=dict()
        allowedpath = os.path.join(dirutil.get_mast_install_path(), 'MAST',
                        'ingredients','programkeys','phon_allowed_keywords.py')
        allowed_list = self._phon_inphon_get_allowed_keywords(allowedpath)
        for key, value in self.keywords['program_keys'].iteritems():
            if not key[0:5] == "mast_":
                keytry = key.upper()
                if not (keytry in allowed_list):
                    self.logger.warning("Ignoring program key %s for INPHON. To allow this keyword, add it to %s" % (keytry, allowedpath))
                else:
                    if type(value)==str and value.isalpha():
                        inphon_dict[keytry]=value.capitalize() #First letter cap
                    else:
                        inphon_dict[keytry]=value
        return inphon_dict

    def _phon_inphon_get_allowed_keywords(self, allowedpath):
        """Get allowed PHON keywords.
            Args:
                allowedpath <str>: file path for allowed PHON keywords
        """
        allowed = MASTFile(allowedpath)
        allowed_list=list()
        for entry in allowed.data:
            allowed_list.append(entry.strip())
        return allowed_list

    def _phon_inphon_setup(self):
        """Set up the INPHON file."""
        name=self.keywords['name']
        myd = dict()
        myd = self._phon_inphon_get_non_mast_keywords()
        my_inphon = MASTFile()
        for key, value in myd.iteritems():
            my_inphon.data.append(str(key) + "=" + str(value).upper() + "\n")
        if not ("NTYPES" in myd.keys()) and not ("MASS" in myd.keys()):
            [nline,massline] = self._phon_inphon_get_masses()
            my_inphon.data.append(nline + "\n")
            my_inphon.data.append(massline + "\n")
        my_inphon.to_file(name + "/INPHON")
        return 

    def _phon_inphon_get_masses(self):
        """Get the ntypes and masses line for INPHON.
            Returns:
                nline, massline
                nline <str>: NYTPES = <number of atomic types in POSCAR>
                massline <str>: MASS = <mass1> <mass2> ...
        """
        name=self.keywords['name']
        if os.path.isfile(name + "/POSCAR_prePHON"):
            mypos = Poscar.from_file(name + "/POSCAR_prePHON")
        else: #not yet modified to strip out the species line.
            mypos = Poscar.from_file(name + "/POSCAR")
        sitesym = mypos.site_symbols
        nline="NTYPES=" + str(len(sitesym))
        massline="MASS="
        for sym in sitesym:
            el = pymatgen.core.periodic_table.Element(sym)
            massline = massline + str(float(el.atomic_mass)) + " "
        return [nline, massline]

    def _phon_forces_setup(self):
        """Set up the FORCES file. This is like the DYNMAT but with the mass
            line stripped out and no direction indicators. Also, a block must
            be present for every atom, with a displacement, even if all entries
            are zero (e.g. fake block for selective dynamics). First line contains
            only the number of total displacements.
        """
        self._replace_my_displacements()
        self._nosd_my_dynmat()
        name=self.keywords['name']
        if not os.path.isfile(name + "/DYNMAT_mod_2"):
            raise MASTError("checker/phon_checker", "No DYNMAT_mod_2 found in %s." % name)
        myvc = VaspChecker(name=self.keywords['name'],program_keys = self.keywords['program_keys'], structure = self.keywords['structure'])
        mydyn=myvc.read_my_dynamical_matrix_file(name, "DYNMAT_mod_2")
        myvc.write_my_dynmat_without_disp_or_mass(mydyn, name, "FORCES")

    def _nosd_my_dynmat(self):
        """Creates fake blocks in DYNMAT for filling back in the atoms and 
            directions skipped through selective dynamics.
        """
        name=self.keywords['name']
        if not os.path.isfile(name + "/DYNMAT_mod_1"):
            raise MASTError("checker/phon_checker", "No DYNMAT_mod_1 found in %s." % name)
        myvc = VaspChecker(name=self.keywords['name'],program_keys = self.keywords['program_keys'], structure = self.keywords['structure'])
        myforces=myvc.read_my_dynamical_matrix_file(name,"DYNMAT_mod_1")
        numatoms = myforces['numatoms']
        myforces['numdisp'] = numatoms * 3 #full set of all blocks
        for atom in range(1, numatoms+1):
            if not atom in myforces['atoms'].keys():
                myforces['atoms'][atom]=dict()
            for dispct in range(1, 4):
                if not dispct in myforces['atoms'][atom].keys():
                    myforces['atoms'][atom][dispct]=dict()
                    if dispct == 1:
                        displine = "0.0001 0 0"
                    elif dispct == 2:
                        displine = "0 0.0001 0"
                    else:
                        displine = "0 0 0.0001"
                    myforces['atoms'][atom][dispct]['displine']=displine
                    myforces['atoms'][atom][dispct]['dynmat']=list()
                    for act in range(0, numatoms):
                        myforces['atoms'][atom][dispct]['dynmat'].append("0.000 0.000 0.000\n")
        myvc = VaspChecker(name=self.keywords['name'],program_keys = self.keywords['program_keys'], structure = self.keywords['structure'])

        myvc.write_my_dynamical_matrix_file(myforces, name, "DYNMAT_mod_2")

    def _replace_my_displacements(self):
        """
            In VASP, 0.01 0 0 in DYNMAT from phonons is 1/Angstroms in the 
            x-direction (XDATCAR shows that it makes fractional coord 
            displacements for all 3 lattice vectors in a non-cubic system to get 
            this strictly-x-direction) 
            In PHON, 0.01 0 0 means 0.01 multiplied by lattice vector a.
            Back out the fractional displacements used by VASP from the XDATCAR, 
            match them up, and use them.
            Konfig =1 is the un-displaced cell.
            Now for NFREE=2,
            there are two Konfigs for each displacment; the first is positive
            POTIM in the x-direction (for example, POTIM = 0.01), then negative
            POTIM in the x-direction, then y, then z.
            So one unfrozen atom has seven Konfigs.
            DYNMAT, however, reports the AVERAGE force from each Konfig pair.
            So we only want Konfigs 2, 4, and 6, corresponding to POTIM 0 0, 
            0 POTIM 0, and 0 0 POTIM
        """
        name=self.keywords['name']
        if not os.path.isfile(name + "/XDATCAR"):
            raise MASTError("checker/phon_checker", "No XDATCAR found in %s." % name)
        myvc = VaspChecker(name=self.keywords['name'],program_keys = self.keywords['program_keys'], structure = self.keywords['structure'])
        myxdat=myvc.read_my_displacement_file(name)
        if not os.path.isfile(name + "/DYNMAT"):
            raise MASTError("checker/phon_checker", "No DYNMAT found in %s." % name)
        myforces=myvc.read_my_dynamical_matrix_file(name)
        atomlist = myforces['atoms'].keys()
        atomlist.sort()
        #first disp needs kfg 2
        #second disp needs kfg 4
        #third disp needs kfg 6...
        dispct=0
        for atom in atomlist:
            displist = myforces['atoms'][atom].keys()
            displist.sort()
            for disp in displist:
                dispct = dispct + 1
                kfgidx = dispct * 2
                atomline = myxdat['configs'][kfgidx][atom-1] #indexing of atoms starts at config list entry 0 for atom 1
                baseline = myxdat['configs'][1][atom-1]
                atomcoords = np.array(atomline.strip().split(), float)
                basecoords = np.array(baseline.strip().split(), float)
                dispcoords = atomcoords - basecoords
                displine = str(dispcoords[0]) + " " + str(dispcoords[1]) + " " + str(dispcoords[2])
                myforces['atoms'][atom][disp]['displine'] = displine
        myvc.write_my_dynamical_matrix_file(myforces, name, "DYNMAT_mod_1")

    def set_up_program_input(self):
        self._phon_poscar_setup()
        self._phon_inphon_setup()
        self._phon_forces_setup()
        return
    def is_started(self):
        """See if the ingredient has been started on
            the queue.
        """
        if os.path.isfile(os.path.join(self.keywords['name'],'QPOINTS')):
            return True
        else:
            return False
        return



import pymatgen as mg
from pymatgen.core.structure import Structure

filename=raw_input('file name:')
l=raw_input('x scale:')
m=raw_input('y scale:')
n=raw_input('z scale:')

struct=mg.read_structure(filename)
Structure.make_supercell(struct,[l,m,n])
mg.write_structure(struct,"poscar_temp")

f=open("poscar_temp")
w=open("mast.inp","w")

def getinfo(line):
    line=line.strip('\n')
    data=line.split(' ')
    while 1:
        try: data.remove('')
        except: break
    return data

line=[]
elenum=[]

line.append(f.readline())
while line[len(line)-1]:
    line.append(f.readline())

i=0
while i<len(line):
    if getinfo(line[i])==[]:
        line.remove(line[i])
        i=i-1
    i=i+1

w.write('begin elementmap\n')

data=getinfo(line[5])   

for j in range(len(data)):
    w.write('X'+str(j+1)+' '+data[j]+'\n')
w.write('end\n\n')
   
w.write('begin lattice\n')

for i in range(2,5):
    data=getinfo(line[i]) 
    w.write(data[0]+' '+data[1]+' '+data[2]+'\n')

w.write('end\n\n')

data=getinfo(line[6])

for j in range(len(data)):
    elenum.append(int(data[j]))
          
w.write('begin coordinates\n')
k=8
for i in range(len(elenum)):
    for j in range(elenum[i]):
        data=getinfo(line[k])
        k=k+1
        w.write('X'+str(i+1)+' '+data[0]+' '+data[1]+' '+data[2]+'\n')
w.write('end\n')


f.close()
w.close()






##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Glen Jenness
# Last updated: 2013-07-03
##############################################################
import sys

"""Helper function for handling errors within MAST
   Modified from:
       http://www.doughellmann.com/articles/how-tos/python-exception-handling/index.html

   Example of use:
        error = 'File %s does not exist' % filename
        MASTerror(self.__class__.__name__, error)
"""

#def MASTError(classname, errormsg):
#    try:
#        raise RuntimeError('%s' % errormsg)
#        return 0
#    except Exception, err:
#        message = '\n\nError detected in MAST class %s!\n' % classname
#        message += 'Please fix the error and rerun mast.\n'
#        message += 'Error message:\n'
#        message += '    %s\n' % errormsg
#        message += '\n\n'
#
#        sys.stderr.write(message)
#        return 1

class MASTError(Exception):
    """Base class for exceptions in MAST."""
    def __init__(self, classname, errormsg):
        message = '\n\nError detected in MAST class %s!\n' % classname
        message += 'Please fix the error and rerun MAST.\n'
        message += 'Error message:\n'
        message += '    %s\n' % errormsg
        message += '\n\n'
        self.value = message
        sys.stderr.write(message)
    def __str__(self):
        return self.value

class FileNotFoundError(MASTError):
    """Exception raised for file not found in MAST."""

    def __init__(self, classname, filepath):
        errormsg = "File not found at " + filepath
        MASTError.__init__(self, classname, errormsg)




import matplotlib.pyplot as plt
import os
import numpy
from MAST.structopt.post_processing.read_genealogy import read_genealogy
import logging

def read_output(folder,genealogytree=False,natoms=None, loggername=None):
    """Function to analyze structural optimization output.
    Inputs:
        folder = folder containing output from structural optimization
        genealogytree = Boolean for whether or not to construct genealogy tree from data
            Default = False
        natoms = number of atoms in an individual 
            Default = False
        loggername = name of log file to write to
    Outputs:
        Plot-All_Energies-XXX.png
        Plot-Peratom-All_Energies-XXX.png
        Plot-Summary-XXX.png
        Plot-time-Summary-XXX.png
        Plot-FPMin-FingerprintMin-XXX-genXX.png
        Plot-FpDist-Fingerprints-XXX-genXX.png
        Plot-SuccessfulMutations-XXX.png
        genealogytree-Genealogy-XXX.png
        
    """
    if loggername:
        #logger = initialize_logger(loggername)
        logger = logging.getLogger(Optimizer.loggername)
    files=os.listdir(folder)
    for filename in files:
        if 'All_Energies' in filename:
            if loggername:
                logger.info('Reading from All_Energies file.  Plotting all energies vs. generation')
            f=open(os.path.join(folder,filename),'r')
            line0=f.readline()
            indenergies=[[float(value)] for value in line0.split()]
            for line in f.readlines():
                i=0
                for value in line.split():
                    a=float(value)
                    if a != 0:
                        indenergies[i].append(float(value))
                    else:
                        indenergies[i].append(None)
                    i+=1
    
            f.close()
            generation=range(len(indenergies[0]))
            clist=[]
            for i in range(len(indenergies)):
                clist.append(numpy.random.rand(3,1))
    
            fig=plt.figure()
            ax1=fig.add_subplot(111)
            i=0
            for i in range(len(indenergies)):
                ax1.scatter(generation,indenergies[i],c=clist[i],label='Ind '+repr(i+1))
    
            handles, labels = ax1.get_legend_handles_labels()
            ax1.legend(handles, labels)
            box = ax1.get_position()
            ax1.set_position([box.x0, box.y0, box.width * 0.8, box.height])
            ax1.legend(loc='center left', bbox_to_anchor=(1, 0.5))
            #ax1.legend(loc='upper center', bbox_to_anchor=(0.5, 1.0),ncol=4, fancybox=True, shadow=True)
            plt.xlabel('Generation')
            plt.ylabel('Total Energy, eV')
            plt.title('Energy Evolution')
            plt.xlim([0,len(generation)])
            plt.savefig('Plot-{0}.png'.format(filename))
            indperatom=[]
            if natoms !=None:
                if loggername:
                    logger.info('Plotting energy per atom vs. generation')
                for ind in indenergies:
                    indperatom.append([en/natoms for en in ind])
        
                fig=plt.figure()
                ax1=fig.add_subplot(111)
                i=0
                for i in range(len(indperatom)):
                    ax1.scatter(generation,indperatom[i],c=clist[i],label='Ind '+repr(i+1))
        
                handles, labels = ax1.get_legend_handles_labels()
                ax1.legend(handles, labels)
                box = ax1.get_position()
                ax1.set_position([box.x0, box.y0, box.width * 0.8, box.height])
                ax1.legend(loc='center left', bbox_to_anchor=(1, 0.5))
                plt.xlabel('Generation')
                plt.ylabel('Energy/atom, eV/atom')
                plt.title('Energy Evolution')
                plt.xlim([0,len(generation)])
                plt.savefig('Plot-Peratom-{0}.png'.format(filename))
        if 'Summary' in filename:
            if 'StructureSummary' not in filename:
                if loggername:
                    logger.info('Plotting Fitness min, max and medium vs. generation')
                f=open(os.path.join(folder,filename),'r')
                line0=f.readline()
                line1=f.readline().split(':')
                bulkepa=float(line1[1])
                line2=f.readline().split()
                natoms=float(line2[1])
                labels=f.readline().split()
                data=dict((lab,[]) for lab in labels)
                for line in f.readlines():
                    i=0
                    for value in line.split(' ',6):
                        if i <6:
                            data[labels[i]].append(float(value))
                        else:
                            sp=value.split("'")
                            data[labels[i]].append(sp[1])
                        i+=1
    
                f.close()
                yl=[data['Fitmedium'][one]-data['Fitmin'][one] for one in range(len(data['Generation']))]
                yh=[data['Fitmax'][one]-data['Fitmedium'][one] for one in range(len(data['Generation']))]
                fig=plt.figure()
                ax1=fig.add_subplot(111)
                ax1.errorbar(data['Generation'],data['Fitmedium'],xerr=0,yerr=[yl,yh])
                plt.xlabel('Generation')
                plt.ylabel('Fitness')
                plt.xlim([1,len(data['Generation'])+1])
                plt.title('Fitness vs. Generation (Medium, Min, Max)')
                plt.ylim([min(data['Fitmin'])-1.0,max(data['Fitmin'])+10.0])
                plt.savefig('Plot-{0}.png'.format(filename))
                srtsl=data['time'][0].split()
                strtime=[float(one) for one in srtsl[3].split(':')]
                tclk=[0.0]
                for one in data['time']:
                    stl=one.split()
                    otime=[float(two) for two in stl[3].split(':')]
                    diff=[otime[i]-strtime[i] for i in range(len(otime))]
                    if diff[0]<0:
                        diff[0]+=24
                    for i in range(len(diff)):
                        if diff[i]<0:
                            diff[i-1]-=1.0
                            diff[i]+=60
                    tclk.append(diff[0]*3600+diff[1]*60+diff[2])
                    strtime=otime
                if loggername:
                    logger.info('Plotting computational time per generation')
                fig = plt.figure()
                ax1 = fig.add_subplot(111)
                ax1.plot(tclk)
                plt.xlabel('Generation')
                plt.ylabel('CPU Time (s)')
                plt.xlim([1,len(data['Generation'])+1])
                plt.title('Processing Time per Generation')
                totaltime=sum(tclk)/3600.0
                plt.text(0.5, 0.95,'Total Time for run = '+'%.3f' % round(totaltime, 3)+'hours',horizontalalignment='center',verticalalignment='top',transform = ax1.transAxes)
                plt.savefig('Plot-time-{0}.png'.format(filename))
        if 'FingerprintMin' in filename:
            if loggername:
                logger.info('Plotting minimum fingerprint functions by generation')
            f = open(os.path.join(folder,filename),'r')
            fpmin=[]
            enfp=[]
            i=0
            for line in f.readlines():
                if i==0:
                    bins=line.split(', ')
                    nbin=[]
                    for one in bins:
                        if '[' in one:
                            one=one[1::]
                        elif ']'in one:
                            ind=[i for i,some in list(enumerate(one)) if some==']']
                            one=one[0:ind[0]]
                        nbin.append(float(one))
                    fpmin.append(nbin)
                    i=1
                elif i==1:
                    enfp.append(float(line))
                    i=0
            f.close()
            for i in range(len(fpmin)):
                fig=plt.figure()
                ax=fig.add_subplot(111)
                ax.plot(fpmin[i])
                plt.xlabel('Distance, bins')
                plt.ylabel('Intensity')
                plt.title('Minimum Individual Fingerprint, Generation={0}'.format(i))
                plt.text(0.5, 0.95,'Structure Energy = '+repr(enfp[i]),horizontalalignment='center',verticalalignment='top',transform = ax.transAxes)
                if max(fpmin[i])>100:
                    plt.ylim([0,100])
                plt.savefig('Plot-FPMin-{0}-gen{1}.png'.format(filename,i))
        if 'Fingerprints' in filename:
            if loggername:
                logger.info('Plotting fingerprint distance vs. energy by generation')
            f = open(os.path.join(folder,filename),'r')
            fpds=[]
            ens=[]
            for line in f.readlines():
                fpg=[]
                eng=[]
                i=0
                for value in line.split():
                    if i==0:
                        fpg.append(float(value))
                        i=1
                    else:
                        eng.append(float(value))
                        i=0
                fpds.append(fpg)
                ens.append(eng)
    
            f.close()
    
            for i in range(len(fpds)):
                emin=min(ens[i])
                eng=[one-emin for one in ens[i]]
                fig=plt.figure()
                ax=fig.add_subplot(111)
                ax.scatter(fpds[i],eng)
                plt.xlabel('Fingerprint Cosine Distance from Minimum Energy Structure')
                plt.ylabel('Energy Difference, eV')
                plt.title('Fingerprint Distance vs. Energy, Generation={0}'.format(i))
                plt.savefig('Plot-FpDist-{0}-gen{1}.png'.format(filename,i))
        if 'Genealogy' in filename:
            if loggername:
                logger.info('Plotting bar plot of mutation success')
            f = open(os.path.join(folder,filename),'r')
            line0=f.readline()
            indhist=[[value] for value in line0.split()]
            for line in f.readlines():
                i=0
                for value in line.split():
                    indhist[i].append(value)
                    i+=1
            mutoptn = ['Lattice_Alteration_nn', 'Lattice_Alteration_rdrd', 'Lattice_Alteration_Group', 'Rotation_geo', 'ZP_Rotation', 'Random_Replacement']
            mutopt = ['LANN','LARD','LAG','GR','ZPR','RGR']
            count = [[0 for j in range(len(indhist[0]))] for i in mutopt]
            for i in range(len(indhist[0])):
                for j in range(len(indhist)):
                    for k in range(len(mutopt)):
                        if mutopt[k] in indhist[j][i]:
                            count[k][i]+=1
            generation=numpy.arange(len(indhist[0]))
            width=1.0/(len(count)+1)
            clist=[]
            for i in range(len(count)):
                clist.append(numpy.random.rand(3,1))

            fig=plt.figure()
            ax1=fig.add_subplot(111)
            for i in range(len(count)):
                ax1.bar(generation+width*i,count[i],width,color=clist[i],label=mutoptn[i])

            plt.xlabel('Generation')
            plt.ylabel('Successful Mutations')
            ax1.legend(loc='upper left')
            plt.title('Successful Mutations by Generation')
            plt.savefig('Plot-SuccessfulMutations-{0}.png'.format(filename))
            if genealogytree:
                if loggername:
                    logger.info('Plotting genealogy tree')
                read_genealogy(filename)
        if 'Bests-energies' in filename:
            if loggername:
                logger.info('Plotting Best-energies scatter')
            f=open(os.path.join(folder,filename),'r')
            benergies = []
            for line in f.readlines():
                benergies.append(float(line))    
            f.close()
            fig=plt.figure()
            ax1=fig.add_subplot(111)
            ax1.scatter(range(len(benergies)),benergies)
            plt.xlabel('Rank')
            plt.ylabel('Total Energy, eV')
            plt.title('Energy of Best Structures in Optimization')
            plt.savefig('Plot-{0}.png'.format(filename))
        else:
            pass




##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Glen Jenness
# Last updated: 2013-07-01
##############################################################

#import numpy as np

#import os
#import warnings
from MAST.utility.masterror import MASTError


class MASTObj(object):
    """Base MAST class.  Used to handle keyword input into MAST classes in a 
    uniform fashion.

    For examples see:
        MAST/interface/interface
        MAST/defects/defecttype

    Attributes:
        allowed_keys <dict>: Each key in this dictionary is a keyword for that
                             class, with each keyvalue being a tuple containing
                             the keyword type, default value, and doc string
        keywords <dict>: The **kwargs passed into the class
    """
    def __init__(self, allowed_keys, **kwargs):
        """
        Initializer for MASTobj.
        Calls:
           self.set_keywords

        Sets:
           self.keywords
           self.allowed_keys

        Returns:
           Nothing 
        """
        self.allowed_keys = allowed_keys
        self.keywords = self.set_keywords(**kwargs)

    def set_keywords(self, **kwargs):
        """
        Takes the self.allowed_keys dictionary, and sets appropiate defaults
        for keywords given in **kwargs and checks to make sure all keywords in
        **kwargs are valid

        Calls:
            Nothing

        Sets:
            Nothing

        Returns:
            Keywords
        """
        keywords = dict()

        for key, value in self.allowed_keys.items():
            keywords[key] = value[1]

        for key, value in kwargs.items():
            if key not in self.allowed_keys:
                error = 'Keyword %s for %s object not found' % \
                                   (key, self.__class__.__name__)
                MASTError(self.__class__.__name__, error)

#                raise RuntimeError('Keyword %s for %s object not found' % \
#                                   (key, self.__class__.__name__))

            if isinstance(value, self.allowed_keys[key][0]):
                keywords[key] = value
            else:
                error = 'Keyword %s value %s invalid; expected type %s, got type %s' % (key, str(value), self.allowed_keys[key][0], type(value))
                MASTError(self.__class__.__name__, error)
#                raise RuntimeError('Keyword %s value invalid' % key)

        return keywords

    def help(self, keyword):
        """Prints a help message listing all the allowed keywords and any relevant information"""
        if (keyword == 'all'):
            string = ('%-20s%-20s%-20s%s\n' % ('Keyword', 'Type', 'Default', 'Comment'))
            for key, value in self.allowed_keys.items():
                string += ('%-20s%-20s%-20s%s\n' % (key, str(value[0]), str(value[1]), value[2]))
            print string




import os
from operator import attrgetter
from ase import Atom, Atoms
from MAST.structopt.inp_out.write_xyz import write_xyz
import logging
try:
    from mpi4py import MPI
except:
    pass

def BestInds(pop, bests, Optimizer, writefile=False, fname=None):
    """Function to keep track of the best individuals throughout an optimization
    Inputs:
        pop = list of Individual class structures to be compared
        bests = list of previously obtained best structures
        Optimizer = Optimizer class structure that contains key parameters
        writefile = True/False boolean to tell function whether or not to output structures
        fname = filename to output structures
    Outputs:
        bests = list of Individual class structures updated with new options
    """
    #logger = initialize_logger(Optimizer.loggername)
    logger = logging.getLogger(Optimizer.loggername)
    logger.info('best_inds_list recieved population with length = {0} and best list with length = {1} for generation {2}'.format(
        len(pop),len(bests),Optimizer.generation))
    pop = sorted(pop, key=attrgetter('fitness'))
    tol=Optimizer.demin
    if len(bests)!=0:
        bfits=[ind.fitness for ind in bests]
        #Identify the fitnesses in the population
        popfits=[ind.fitness for ind in pop]
        #Initialize a list for individuals to not be added to best list
        indices=[]
        for i in range(len(popfits)):
            for j in range(len(bfits)):
                if popfits[i]==bfits[j]:
                    indices.append(i)
                if popfits[i] < bfits[j]:
                    diff = abs(bfits[j] - popfits[i])
                    diff2 = abs(bfits[j-1] - popfits[i])
                    if diff > tol:
                        if diff2 > tol:
                            if i not in indices:
                                bests.append(pop[i])
                                logger.info('Best list found new structure for List HI = {0}'.format(pop[i].history_index))
                                indices.append(i)
                                b=bfits[0:j]
                                b.append(pop[i].fitness)
                                b.extend(bfits[j::])
                                bfits=b
                        else:
                            indices.append(i)
        bests = sorted(bests, key=attrgetter('fitness'))
        if len(bests) < Optimizer.number_of_bests:
            bmax = bests[len(bests)-1].fitness
            for i in range(len(popfits)):
                if popfits[i] > bmax:
                    diff = abs(popfits[i]-bmax)
                    if diff > tol:
                        bests.append(pop[i])
                        logger.info('Appending best list with lower energy structure')
                        bmax = pop[i].fitness
        else:
            logger.info('Removing extra {0} structures from best list'.format(len(bests)-Optimizer.number_of_bests))
            bests = bests[0:Optimizer.number_of_bests]
        bfits=[ind.fitness for ind in bests]
    else:
        bests = []
        bfits = []
        for one in pop:
            fit = one.fitness
            if len(bfits) == 0:
                bests.append(one)
                logger.info('Adding new structure to best list HI = {0}'.format(one.history_index))
                bfits.append(fit)
            else:
                diff = abs(bfits[-1]-fit)
                if diff > tol:
                    bests.append(one)
                    bfits.append(fit)
        if len(bests) > Optimizer.number_of_bests:
            logger.info('Removing extra {0} structures from best list'.format(len(bests)-Optimizer.number_of_bests))
            bests = bests[0:Optimizer.number_of_bests]
    if writefile==True:
        if fname==None:
            try:
                rank = MPI.COMM_WORLD.Get_rank()
            except:
                rank = 0
            path = os.path.join(os.getcwd(),'{0}-rank{1}'.format(Optimizer.filename,rank))
            logger.info('Writing Best lists to {0}'.format(path))
            fxyzname=os.path.join(path,'Bests-{0}.xyz'.format(Optimizer.filename))
            ffitname=os.path.join(path,'Bests-fitnesses-{0}.txt'.format(Optimizer.filename))
        else:
            fxyzname='{0}.xyz'.format(fname)
            ffitname='{0}-fitnesses.txt'.format(fname)
        bestfile=open(fxyzname,'w')
        bestfitfile=open(ffitname,'w')
        for one in bests:
            if Optimizer.structure=='Defect':
                cbulk=Atoms()
                cbulk.extend(one[0])
                cbulk.extend(one.bulki)
                write_xyz(bestfile,cbulk,one.energy)
            else:
                write_xyz(bestfile,one[0],one.energy)
            bestfitfile.write(repr(one.fitness)+'\n')
        bestfile.close()
        bestfitfile.close()
    return bests
    


import random
import copy

def cxtpc(ind1, ind2, Optimizer):
    """ Simple Two point Crossover for crystals
    Positions exchanged as fraction of cell
    Maintains concentration
    Maintains total number of atoms
    Random clusters not necessarily geometry based
    """
    if 'CX' in Optimizer.debug:
        debug = True
    else:
        debug = False
    Optimizer.output.write('Two Pt Crystal-Fraction Cx between individual '
        +repr(ind1.index)+' and individual '+repr(ind2.index)+'\n')
    indi1 = ind1[0]
    indi2 = ind2[0]
    pos1 = indi1.get_scaled_positions()
    pos2 = indi2.get_scaled_positions()
    size = min(len(pos1), len(pos2))
    cxpoint1 = random.randint(1, size)
    cxpoint2 = random.randint(1, size - 1)
    if cxpoint2 >= cxpoint1:
        cxpoint2 += 1
    else:
        # Swap the two cx points
        cxpoint1, cxpoint2 = cxpoint2, cxpoint1
    posi1 = copy.deepcopy(pos1)
    posi2 = copy.deepcopy(pos2)
    posi1[cxpoint1:cxpoint2] = pos2[cxpoint1:cxpoint2]
    posi2[cxpoint1:cxpoint2] = pos1[cxpoint1:cxpoint2]
    indi1.set_scaled_positions(posi1)
    indi2.set_scaled_positions(posi2)
    if debug:
        print 'DEBUG CX: CXPT1 = ',cxpoint1,'CXPT2 = ',cxpoint2
    ind1[0] = indi1
    ind2[0] = indi2
    return ind1, ind2




"""Tests for Chopingredient"""

from MAST.ingredients.chopingredient import ChopIngredient
import unittest
from unittest import SkipTest
import os
import time
import MAST
import pymatgen
from MAST.utility import dirutil
from MAST.utility import MASTFile
import shutil

testname="chop_test_complete"
testdir = dirutil.get_test_dir(testname)

class TestIsCompleteIngredient(unittest.TestCase):
    def setUp(self):
        os.chdir(testdir)

        if not os.path.isdir("writedir"):
            os.mkdir("writedir")
        if not os.path.isdir("writedir/single_label1"):
            os.mkdir("writedir/single_label1")

    def tearDown(self):
        tearlist = list()
        tearlist.append("writedir")
        for foldername in tearlist:
            try:
                shutil.rmtree(foldername)
            except OSError:
                pass

    def test___init__(self):
        self.assertTrue(True)
        #raise SkipTest
        #self.testclass.__init__(**kwargs)
    
    def test_complete_structure(self):
        #raise SkipTest
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        kdict['mast_kpoints'] = [2,2,2,"M"]
        kdict['mast_xc'] = 'pw91'
        my_pos = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure")
        my_pos.write_file("%s/CONTCAR" % ingdir)
        myci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_pos.structure)
        self.assertTrue(myci.complete_structure())
        os.remove("%s/CONTCAR" % ingdir)
        self.assertFalse(myci.complete_structure())
        #self.testclass.complete_structure()

    def test_complete_singlerun(self):
        #raise SkipTest
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        kdict['mast_kpoints'] = [2,2,2,"M"]
        kdict['mast_xc'] = 'pw91'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywr = ChopIngredient(name=ingdir, program_keys = kdict, structure=my_structure)
        mywr.write_singlerun()
        myoutcar = MASTFile("files/OUTCAR_completed")
        myoutcar.to_file("%s/OUTCAR" % ingdir)
        myoszicar = MASTFile("files/OSZICAR_completed")
        myoszicar.to_file("%s/OSZICAR" % ingdir)
        myrdi = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        self.assertTrue(myrdi.complete_singlerun())
        os.remove("%s/OUTCAR" % ingdir)
        self.assertFalse(myrdi.complete_singlerun())
        #self.testclass.complete_singlerun()


    def test_complete_neb_subfolders(self):
        #raise SkipTest
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        kdict['mast_kpoints'] = [2,2,2,"M"]
        kdict['mast_xc'] = 'pw91'
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images'] = 3
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywr = ChopIngredient(name=ingdir, program_keys = kdict, structure=my_structure)
        myoutcar = MASTFile("files/OUTCAR_completed")
        myoszicar = MASTFile("files/OSZICAR_completed")
        for subdir in ['00','01','02','03','04']:
            subname = "%s/%s" % (ingdir, subdir)
            os.mkdir(subname)
            mywr.keywords['name'] = subname
            mywr.checker.keywords['name'] = subname
            if not subdir in ['00','04']:
                mywr.write_singlerun()
                mywr.write_submit_script()
                myoutcar.to_file("%s/OUTCAR" % subname)
                myoszicar.to_file("%s/OSZICAR" % subname)
        myci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        self.assertTrue(myci.complete_neb_subfolders())
        os.remove("%s/03/OUTCAR" % ingdir)
        self.assertFalse(myci.complete_neb_subfolders())
        #self.testclass.complete_neb_subfolders()

    def test_complete_subfolders(self):
        #raise SkipTest
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        kdict['mast_kpoints'] = [2,2,2,"M"]
        kdict['mast_xc'] = 'pw91'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywr = ChopIngredient(name=ingdir, program_keys = kdict, structure=my_structure)
        myoutcar = MASTFile("files/OUTCAR_completed")
        myoszicar = MASTFile("files/OSZICAR_completed")
        for subdir in ['00','01','02','03','04']:
            subname = "%s/%s" % (ingdir, subdir)
            os.mkdir(subname)
            mywr.keywords['name'] = subname
            mywr.checker.keywords['name'] = subname
            mywr.write_singlerun()
            mywr.write_submit_script()
            myoutcar.to_file("%s/OUTCAR" % subname)
            myoszicar.to_file("%s/OSZICAR" % subname)
        myci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        self.assertTrue(myci.complete_subfolders())
        os.remove("%s/04/OUTCAR" % ingdir)
        self.assertFalse(myci.complete_subfolders())
        #self.testclass.complete_subfolders()
        





##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-05-12 by Zhewen Song
##############################################################
from pymatgen.io.vaspio import *
import numpy as np
import logging
from MAST.utility.dirutil import *
from MAST.utility import MASTError
from MAST.utility import MASTFile
from MAST.utility import MASTObj
from MAST.utility import loggerutils
from pymatgen.core.structure import Structure
from pymatgen.util.coord_utils import find_in_coord_list
from pymatgen.util.coord_utils import find_in_coord_list_pbc
from pymatgen.core.sites import PeriodicSite

class StructureExtensions(MASTObj):
    """Structure extensions
    """
    def __init__(self, **kwargs):
        allowed_keys = {
            'struc_work1': (Structure, None, 'First working Pymatgen Structure object (e.g. create a defect, or use work1 and work2 to interpolate positions)'),
            'struc_work2': (Structure, None, 'Second working Pymatgen Structure object'),
            'struc_init': (Structure, None, 'Initial structure at the beginning of the MAST recipe'),
            'scaling_size':(str,None,'Scaling size'),
            'name': (str, os.getenv("MAST_CONTROL"), 'Name of ingredient')
            }
        MASTObj.__init__(self, allowed_keys, **kwargs)
        self.logger = logging.getLogger(self.keywords['name'])
        self.logger = loggerutils.add_handler_for_recipe(self.keywords['name'], self.logger)
        self.metafile = Metadata(metafile='%s/metadata.txt' % self.keywords['name'])

    def induce_defect(self, defect, coord_type, threshold):
        """Creates a defect, and returns the modified structure
            Args:
                defect <dict>: Defect subdictionary (single 
                               defect) of the form:
                        {'symbol': 'cr', 'type': 'interstitial', 
                    'coordinates': array([ 0. ,  0.5,  0. ])}}
                coord_type <str>: cartesian or fractional
                threshold <float>: Threshold for finding the
                                   defect position in what may
                                   be a relaxed, imperfect 
                                   structure.
            Returns:
                defected structure <Structure>
        """
        struct_ed = self.keywords['struc_work1'].copy() 
        symbol = defect['symbol'].title() #Cap first letter

        # If we have cartesian coordinates, then we convert them to fractional here.
        if ('cartesian' in coord_type):
            defect['coordinates'] = self._cart2frac(defect['coordinates'], self.keywords['struc_work1'])

        if (defect['type'] == 'vacancy'):
            self.logger.info('Creating a %s vacancy at %s' % (symbol, str(defect['coordinates'])))
            index = find_in_coord_list_pbc(self.keywords['struc_work1'].frac_coords,
                                       defect['coordinates'],
                                       atol=threshold)
            if len(index) > 1:
                raise MASTError(self.__class__.__name__, "Multiple indices %s found. Check structure and/or adjust threshold %s to finer tolerance for ingredient %s" % (index, threshold,self.keywords['name']))
            if len(index) == 0:
                raise MASTError(self.__class__.__name__, "No indices found. Check structure and/or adjust threshold %s to lower tolerance for ingredient %s" % (threshold,self.keywords['name']))
            struct_ed.remove_sites([index])
        elif (defect['type'] == 'interstitial'):
            self.logger.info('Creating a %s interstitial at %s' % (symbol, str(defect['coordinates'])))
            struct_ed.append(symbol,
                                  defect['coordinates'],
                                  coords_are_cartesian=False,
                                  validate_proximity=True)
        elif (defect['type'] in ['antisite', 'substitution']):
            self.logger.info('Creating a %s antisite at %s' % (symbol, str(defect['coordinates'])))
            index = find_in_coord_list_pbc(self.keywords['struc_work1'].frac_coords,
                                       defect['coordinates'],
                                       atol=threshold)
            if len(index) > 1:
                raise MASTError(self.__class__.__name__, "Multiple indices %s found. Check structure and/or adjust threshold %s to finer tolerance for ingredient %s" % (index, threshold, self.keywords['name']))
            if len(index) == 0:
                raise MASTError(self.__class__.__name__, "No indices found. Check structure and/or adjust threshold %s to lower tolerance for ingredient %s" % (threshold, self.keywords['name']))
            struct_ed.replace(index, symbol)
            struct_ed = struct_ed.get_sorted_structure()
        else:
            raise RuntimeError('Defect type %s not supported' % defect['type'])

        return struct_ed
    def _cart2frac(self, position, base_structure):
        """Converts between cartesian coordinates and fractional coordinates"""
        fractional =  base_structure.lattice.get_fractional_coords(position)
        for i in range(len(fractional)):
            if (fractional[i] < 0.0):
                fractional[i] += 1.0
            elif (fractional[i] > 1.0):
                fractional[i] -= 1.0
        return fractional

    def sort_structure_and_neb_lines(self, neblines, folderstr, images=0):
        """Sort the structure in an approved way:
            1. Remove lines which closely match the neb moving 
               lines in the NEB section.
            2. Sort the structure by element and coordinate.
            3. Prepend the NEB moving lines to their element 
               sections.
            4. Return the modified structure.
            Args:
                folderstr <str>  : '00' = initial config, 
                                   '0N+1' = final config,
                                   '0N' = corresponding image
                neblines <list>  : list of NEB lines
                images <int>     : number of images
            Returns:
                sorted structure <Structure>
        """
        import MAST.data
        atol = 0.1 # Need fairly large tolerance to account for relaxation.
        sortedstruc = self.keywords['struc_work1'].get_sorted_structure()
        struct_ed = sortedstruc.copy()
        nebidx = list()
        elemstarts = self._get_element_indices(sortedstruc)
        for nebline in neblines:
            nebdict = self._parse_neb_line(nebline)
            temp_fin = sortedstruc.copy()
            temp_fin.append(nebdict['element'],nebdict['coord'][1])
            temp_start = sortedstruc.copy()
            temp_start.append(nebdict['element'],nebdict['coord'][0])
            strlist=temp_start.interpolate(temp_fin, images+1)
            lastidx = strlist[0].num_sites-1
            if folderstr == '00':
                mycoord = nebdict['coord'][0]
            elif folderstr == str(images+1).zfill(2):
                mycoord = nebdict['coord'][1]
            else:
                mystridx = int(folderstr)
                mycoord = strlist[mystridx].frac_coords[lastidx]

            scalingsize = self.metafile.read_data('scaling_size')
            if not (scalingsize == None):
                scalingsize = np.array(scalingsize.split('x'),float)
                for i in range(3):
                    mycoord[i] /= scalingsize[i]

            indexraw = find_in_coord_list_pbc(sortedstruc.frac_coords, mycoord, atol)
            index=list()
            for indexentry in indexraw:
                if sortedstruc.species[indexentry] == temp_start.species[lastidx]:
                    index.append(indexentry)
            if len(index) == 0:
                raise MASTError("pmgextend/structure_extensions", "No coordinate found matching %s for %s" % (mycoord, self.keywords['name']))
            nebidx.append(index[0]) #only take first site?
            mysite = sortedstruc.sites[index[0]]
            myelem = MAST.data.atomic_number[mysite.species_string]
            struct_ed.remove_sites([index[0]])
            struct_ed.insert(elemstarts[myelem], mysite.specie,
                                    mysite.frac_coords)
            sortedstruc = struct_ed.copy() #get new ordering
        if not len(nebidx) == len(neblines):
            raise MASTError("pmgextend/structure_extensions", "Not all NEB lines found for %s" % self.keywords['name'])
        return struct_ed

    def _get_element_indices(self, sortedstruc):
        """From a sorted structure, get the element indices
            Args:
                sortedstruc <Structure>: pymatgen structure 
                                sorted by electronegativity
            Returns:
                elstart <dict>: element index dictionary, in the
                                format:
                elstart[element number] = starting index, e.g.
                elstart[24] = 0
        """
        elstart = dict()
        numlist = sortedstruc.atomic_numbers
        listlen = len(numlist)
        idx = 0
        while idx < listlen:
            atomnum = numlist[idx]
            if not atomnum in elstart.keys():
                elstart[atomnum] = idx
            idx = idx + 1
        return elstart

    def _parse_neb_line(self, nebline):
        """Parse an NEB atomic movement line which has the 
           following format:
                    Cr, 0 0 0, 0.5 0.5 0.5 in the input file, and
                    ['Cr','0 0 0','0.5 0.5 0.5'] here as nebline
                    element, init_coord, fin_coord
            Args:
                line <list of str>: NEB atomic movement line
            Returns:
                linedict <dict>:
                    ['element'] <int> = element's atomic number
                    ['coord'][0] <np.array> = initial coordinates
                    ['coord'][1] <np.array> = final coordinates
        """
        nebdict=dict()
        nebelem = str(nebline[0].strip()).title()
        import MAST.data
        nebdict['element'] = MAST.data.atomic_number[nebelem]
        nebdict['coord'] = dict()
        nebdict['coord'][0] = np.array(nebline[1].split(), dtype='float')
        nebdict['coord'][1] = np.array(nebline[2].split(), dtype='float')
        return nebdict


    def do_interpolation(self, numim):
        """Do interpolation.
            Both struc_work1 and struc_work2 should be defined.
            Args:
                numim <int>: number of images
            Returns:
                <list of Structure>: list of interpolated 
                 structures, including the two endpoints
        """
        if self.keywords['struc_work1'] == None:
            raise MASTError(self.__class__.__name__, "No initial state structure for %s" % self.keywords['name'])
        if self.keywords['struc_work2'] == None:
            raise MASTError(self.__class__.__name__, "No final state structure for %s" % self.keywords['name'])
        structure_list = self.keywords['struc_work1'].interpolate(self.keywords['struc_work2'], numim+1)
        return structure_list

    def get_sd_array(self, phonon_center_site, phonon_center_radius, threshold=1e-1):
        """Create a selective dynamics array.
            Args:
                phonon_center_site <str>: phonon center site (coordinate)
                phonon_center_radius <float>: phonon center radius
                threshold <float>: absolute matching threshold for coordinates; 
                    e.g. if this value is 0.1 then both (0.1,0,0) and (0.3,0,0) 
                    will match for a search on center_site of (0.2,0,0)
        """
        mynbarr = self._get_neighbor_array(phonon_center_site, phonon_center_radius, self.keywords['struc_work1'], threshold)
        mysd = np.zeros([self.keywords['struc_work1'].num_sites,3],bool)
        for myn in mynbarr:
            mysd[myn]=np.ones(3,bool)
        return mysd

    def _get_neighbor_array(self, phonon_center_site, phonon_center_radius, mystruc, tol=1e-1):
        """
            Get a neighbor-index array.
            Use program_keywords 'phonon_center_site' and
            'phonon_center_radius' to limit the number of phonons calculated.
            Args:
                phonon_center_site <str>: phonon center site (coordinate)
                phonon_center_radius <float>: phonon center radius in Angstroms. If nonzero, all atoms in a radius around EACH site found in phonon_center_site will also be taken into account
                mystruc <Structure>: pymatgen Structure
                tol <float>: Tolerance for match-searching.
        """
        if phonon_center_site == None:
            return None
        pcscoord = np.array(phonon_center_site.strip().split(), float)
        scalingsize = self.metafile.read_data('scaling_size')
        if not (scalingsize == None):
            scalingsize = np.array(scalingsize.split('x'),float)
            for i in range(3):
                pcscoord[i] /= scalingsize[i]
        tol = float(tol)
        pcsarr = find_in_coord_list_pbc(mystruc.frac_coords, pcscoord,tol)
        uniqsites = np.unique(pcsarr)

        if len(uniqsites) == 0:
            raise MASTError("pmgextend/structure_extensions", "No sites found for phonon centering for %s" %self.keywords['name'])

        if phonon_center_radius == None:
            return uniqsites

        nrad = float(phonon_center_radius)
        if nrad == 0:
            return uniqsites
        if nrad < 0:
            raise MASTError("pmgextend/structure_extensions", "Phonon center radius should not be less than zero for %s!" % self.keywords['name'])

        nbtotarr=None
        for pcs in uniqsites:
            neighbors = mystruc.get_neighbors(mystruc[pcs], nrad, True)
            if nbtotarr == None:
                nbtotarr = neighbors
            else:
                np.concatenate([nbtotarr, neighbors])
        nbsitelist=list()
        for nbr in nbtotarr:
            nbsitelist.append(nbr[-1])
        nbsitelist = np.array(nbsitelist)
        alltotarr = np.concatenate([uniqsites, nbsitelist])
        allsites = np.unique(alltotarr)
        return allsites

    def get_multiple_sd_array(self, phonon_center_site, phonon_center_radius,threshold=1e-1):
        """Create a selective dynamics array, for use when every atom and every
            direction is a separate calculation (T F F, etc.)
            Args:
                phonon_center_site <str>: phonon center site (coordinate)
                phonon_center_radius <float>: phonon center radius, in Angstroms
                threshold <float>: absolute matching threshold for coordinates; e.g. if this value is 0.1, then both (0.1,0,0) and (0.3,0,0) will match for a search on center_site of (0.2,0,0)
            Returns:
                mysdlist <list>: list of SD arrays
        """
        if phonon_center_site == None:
            return None
        mynbarr = self._get_neighbor_array(phonon_center_site, phonon_center_radius, self.keywords['struc_work1'], threshold)
        mysdlist=list()
        for myn in mynbarr:
            for myct in range(0,3):
                mysd = np.zeros([self.keywords['struc_work1'].num_sites,3],bool)
                mysd[myn]=np.zeros(3,bool)
                mysd[myn][myct]=1
                mysdlist.append(mysd)
        return mysdlist

    def graft_coordinates_onto_structure(self, coordstruc):
        """Graft coordinates from mast_coordinates Structure objects
            onto the appropriate structure
            Args:
                coordstruc <Structure>: Structure object with
                    the coordinates for grafting
                self.keywords['struc_work1'] will contain
                    the elements and lattice parameter, which
                    will not be touched.
            Returns:
                modified Structure object <Structure>
        """
        goodstruc = self.keywords['struc_work1'].copy()
        lengoodsites=len(goodstruc.sites)
        lencoordsites=len(coordstruc.sites)
        if not (lengoodsites == lencoordsites):
            raise MASTError(self.__class__.__name__, "Original and coordinate structures do not have the same amount of sites in %s" % self.keywords['name'])
        cct=0
        newsites=list()
        mylattice=goodstruc.lattice
        while cct < lengoodsites:
            newcoords=coordstruc.sites[cct].frac_coords
            oldspecie=goodstruc.sites[cct].specie
            newsite=PeriodicSite(oldspecie, newcoords, mylattice)
            newsites.append(newsite)
            cct=cct+1
        goodstruc.remove_sites(range(0,lengoodsites))
        for cct in range(0, lengoodsites):
            goodstruc.append(newsites[cct].specie,
                newsites[cct].frac_coords)
        return goodstruc
    def strain_lattice(self, strainstring):
        """Strain the lattice. 
            Args:
                strainstring <str>: Strain string from 
                    'mast_strain' in program keys, 
                    with space-separated strain along 
                    lattice vectors a, b, and c, e.g.
                    "0.98 0.96 1.01"
            Returns:
                newstructure <Structure>: strained structure
        """
        mystructure = self.keywords['struc_work1']
        strainsplit = strainstring.strip().split()
        strarray = np.array([[0.],[0.],[0.]],'float')
        for sidx in range(0,3):
            strflt = float(strainsplit[sidx])
            strarray[sidx][0]=strflt
        newlattice = Lattice(np.multiply(mystructure._lattice.matrix, strarray)) #be very careful here. np.multiply is NOT regular matrix multiplication.
        newstructure = mystructure.copy()
        newstructure.modify_lattice(newlattice)
        return newstructure

    def scale_structure(self):
        scaledstr = self.keywords['struc_work1'].copy()
        scale = self.keywords['scaling_size']
        scale = scale.strip()
        scalesplit = scale.split()
        if len(scalesplit) == 3:
            scaleinput = map(float, scalesplit)
        else:
            self.logger.error("Wrong number of inputs for scaling: %s " % scalesplit)
            raise MASTError("Wrong number of inputs for scaling: %s " % scalesplit)
            return None
        scaledstr.make_supercell(scaleinput)
        return scaledstr

    def scale_defect(self, defect, coord_type, threshold):
        """Scales the defect dictionary and returns the modified structure
            Args:
                defect <dict>: Defect subdictionary (single 
                               defect) of the form:
                        {'symbol': 'cr', 'type': 'interstitial', 
                    'coordinates': array([ 0. ,  0.5,  0. ])}}
                coord_type <str>: cartesian or fractional
                threshold <float>: Threshold for finding the
                                   defect position in what may
                                   be a relaxed, imperfect 
                                   structure.
            Returns:
                defected structure <Structure>
        """
        mycoords = defect['coordinates']
        scale = self.keywords['scaling_size']
        scale = scale.strip()
        scalesplit = scale.split()
        if len(scalesplit) == 3:
                scaleinput = map(float, scalesplit)
        else:
            self.logger.error("Wrong number of inputs for scaling: %s " % scalesplit)
            raise MASTError("Wrong number of inputs for scaling: %s " % scalesplit)
            return None

        #if newa is twice as big as olda, coordinate should be half as big
        coorda = mycoords[0]/scaleinput[0]
        coordb = mycoords[1]/scaleinput[1]
        coordc = mycoords[2]/scaleinput[2]
        newdict = dict(defect)
        newdict['coordinates'] = np.array([coorda, coordb, coordc],'float')
        returnstr = self.induce_defect(newdict, coord_type, threshold)
        return returnstr



#!/usr/bin/env python
##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
#TTM 2014-03-27
#use pymatgen.phasediagram.pdanalyzer get_decomposition(pymatgen.core.composition Composition, and using an open element O2 quaternary phase diagram) to get compositions and percentages for some composition (e.g. LaMn0.4Fe0.6O3)
from pymatgen.matproj.rest import MPRester
from pymatgen.phasediagram.pdmaker import PhaseDiagram
from pymatgen.phasediagram.pdmaker import GrandPotentialPhaseDiagram
from pymatgen.phasediagram.plotter import PDPlotter
from pymatgen.phasediagram.pdanalyzer import PDAnalyzer
from pymatgen.phasediagram.entries import PDEntry
from pymatgen.core.composition import Composition
from pymatgen.core.periodic_table import Element

def get_decomp(o_chem_pot, mycomp, verbose=1):
    """Get decomposition from open phase diagram
        Args:
            o_chem_pot <float>: Oxygen chemical potential
            mycomp <pymatgen Composition>: Composition
            verbose <int>: 1 - verbose (default)
                           0 - silent
        Returns:
            decomposition string
    """        
    a = MPRester("<YOUR_MPREST_API_KEY_HERE>")
    elements = mycomp.elements
    ellist = map(str, elements)
    entries = a.get_entries_in_chemsys(ellist)
    #entries = a.get_entries_in_chemsys(['La', 'Mn', 'O', 'Fe'])
    pd = PhaseDiagram(entries)
    gppd = GrandPotentialPhaseDiagram(entries,{Element('O'): float(o_chem_pot)})
    print gppd
    #plotter = PDPlotter(gppd)
    #plotter.show()

    gppda = PDAnalyzer(gppd)
    #mychempots = gppda.get_composition_chempots(mycomp)
    #print "My chem pots:"
    #print mychempots
    mydecompgppd = gppda.get_decomposition(mycomp)
    #pdentry = PDEntry(mycomp, 0)
    #print "Decomp and energy:"
    #decompandenergy = gppda.get_decomp_and_e_above_hull(pdentry)
    #print decompandenergy
    #mydecomppd = pda.get_decomposition(mycomp)
    #print "Mn profile:"
    #mnprof= gppda.get_element_profile(Element('Mn'),mycomp)
    #print mnprof

    if verbose:
        for (entry,amount) in mydecompgppd.iteritems():
            print "%s: %3.3f" % (entry.name, amount)
            #mymurangegppd = gppda.getmu_range_stability_phase(Composition(entry.name),Element('O'))
            #print mymurangegppd
        #for (entry,amount) in mydecomppd.iteritems():
        #    print "%s: %3.3f" % (entry.name, amount)
        print ""
    return mydecompgppd

mycomp = Composition(La=1,Mn=0.4,Fe=0.6,O=3)
import numpy as np
import sys
myrange = np.arange(-12,1,0.25)

for ocp in myrange:
    get_decomp(ocp, mycomp)



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Amy Kaczmarowski
# Last updated: 2014-04-25
##############################################################
from pymatgen.core.structure import Structure
from MAST.utility import dirutil
from MAST.utility.mastfile import MASTFile
from MAST.utility import MASTError
from MAST.ingredients.checker import BaseChecker
import os
import logging
import pymatgen
import numpy as np
import time
import subprocess

import ase
import random
from pymatgen.io.aseio import AseAtomsAdaptor
from re import compile as re_compile, IGNORECASE
import decimal as dec
from ase import Atom, Atoms
import shutil

class LammpsChecker(BaseChecker):
    """LAMMPS checker functions
        Mostly structure functions right now.
    """
    def __init__(self, **kwargs):
        allowed_keys = {
            'name' : (str, str(), 'Name of directory'),
            'program_keys': (dict, dict(), 'Dictionary of program keywords'),
            'structure': (Structure, None, 'Pymatgen Structure object')
            }
        BaseChecker.__init__(self, allowed_keys, **kwargs)
        self.potential_file = None

    def get_structure_from_file(self, myfilepath=""):
        """Get the structure from a specified file path.
            For LAMMPS, this is a trajectory-type file.
            Args:
                myfilepath <str>: File path for structure.
        """
        dir = os.path.dirname(myfilepath)
        if dir == '':
            dir = os.getcwd()
        filelist = os.listdir(dir)
        atomsymbolsfile = None
        for onefile in filelist:
            if 'atom_symbols' in onefile:
                atomsymbolsfile = os.path.join(dir, onefile)
        #Read the atom types from the atoms symbol file
        f = open(atomsymbolsfile, 'r')
        atomslist = list()
        for line in f.readlines():
            atomslist.append(line.strip())
        f.close()
        if "DATA" in myfilepath:
            atms = read_lammps_data(myfilepath,atomslist)
        elif "TRAJECTORY" in myfilepath:
            atms,velocities,forces = read_lammps_trajectory(myfilepath,-1,atomslist)
        else:
            raise MASTError(self.__class__.__name__,
                "Unknown file type for receiving structure from LAMMPS: %s" % myfilepath)
        return AseAtomsAdaptor.get_structure(atms)

    def get_initial_structure_from_directory(self,mydir=""):
        """Get the structure from a specified file path.
            For LAMMPS, this is a data-type file.
            Args:
                myfilepath <str>: File path for structure.
        """
        dir = os.path.dirname(myfilepath)
        filelist = os.listdir(dir)
        atomsymbolsfile = None
        for onefile in filelist:
            if 'atom_symbols' in onefile:
                atomsymbolsfile = os.path.join(dir, onefile)
        #Read the atom types from the atoms symbol file
        f = open(atomsymbolsfile)
        atomslist = list()
        for line in f.readlines():
            atomslist.append(line.strip())
        f.close()
        atms = read_lammps_data(myfilepath,atomslist)
        return AseAtomsAdaptor.get_structure(atms)
    
    def get_final_structure_from_directory(self, mydir=""):
        """Get the structure from a specified file path.
            For LAMMPS, this is a trajectory-type file.
            Args:
                myfilepath <str>: File path for structure.
        """
        if mydir == "":
            mydir = self.keywords['name']
        filelist = os.listdir(mydir)
        atomsymbolsfile = None
        for onefile in filelist:
            if 'atom_symbols' in onefile:
                atomsymbolsfile = os.path.join(mydir, onefile)
        #Read the atom types from the atoms symbol file
        f = open(atomsymbolsfile)
        atomslist = list()
        for line in f.readlines():
            atomslist.append(line.strip())
        f.close()
        atms = read_lammps_trajectory(myfilepath,atomslist)
        return AseAtomsAdaptor.get_structure(atms)
    
    def forward_final_structure_file(self, childpath, newname="DATA"):
        """Forward the final structure.
            For LAMMPS, this is the Trajectory.
            Args:
                childpath <str>: Path of child ingredient
                newname <str>: new name (default 'POSCAR')
        """
        trajstruct = self.get_structure_from_file("TRAJECTORY")
        trajatoms = AseAtomsAdaptor.get_atoms(trajstruct)
        write_lammps_data("FINAL",trajatoms)
        self.copy_a_file(childpath, "FINAL", newname)
        self.copy_a_file(childpath, "atom_symbols", "atom_symbols")
    
    def forward_initial_structure_file(self, childpath, newname="POSCAR"):
        """Forward the initial structure.
            For LAMMPS, this is the data file. This function is
            used after phonon calculations, where the CONTCAR
            contains the last displacement. To forward to PHON,
            the POSCAR (without displacements) should be used.
            Args:
                childpath <str>: Path of child ingredient
                newname <str>: new name (default 'POSCAR')
        """
        filelist = os.listdir(childpath)
        dataname = None
        atomsymbolsfile = None
        for onefile in filelist:
            if 'DATA' in onefile:
                dataname = os.path.join(childpath, onefile)
            if 'atom_symbols' in onefile:
                atomsymbolsfile = os.path.join(childpath, onefile)
        if not dataname:
            raise MASTError(self.__class__.__name__,"No trajectory file in %s" % mydir)
        #Read the atom types from the atoms symbol file
        f = open(atomsymbolsfile)
        atomslist = list()
        for line in f.readlines():
            atomslist.append(line.strip())
        f.close()
        atms = read_lammps_data(dataname, atomlist = atomslist)
        from ase.io import read, write
        newpath = os.path.join(childpath, "POSCAR")
        ase.io.write(newpath,aseatomsobj,"vasp", direct=True, sort=True, vasp5=True)
        return self.copy_a_file(childpath, "POSCAR", newname)
    
    def forward_displacement_file(self, childpath, newname="TRAJECTORY"):
        """Forward displacement information.
            For LAMMPS, this is the TRAJECTORY file.
            Args:
                childpath <str>: Path of child ingredient
                newname <str>: new name (default 'TRAJECTORY')
        """
        return self.copy_a_file(childpath, "TRAJECTORY", newname)

    def forward_energy_file(self, childpath, newname="LOG"):
        """Forward the energy file.
            For LAMMPS, this is the LOG file.
            Args:
                childpath <str>: Path of child ingredient
                newname <str>: new name (default 'LOG')
        """
        return self.copy_a_file(childpath, "LOG", newname)

    def is_frozen(self):
        """Check if single LAMMPS non-NEB calculation is frozen.
        """
        return BaseChecker.is_frozen(self, "LOG")
    
    def is_complete(self):
        """Check if single LAMMS non-NEB calculation is complete.
        """
        opath = os.path.join(self.keywords['name'],"LOG")
        if not os.path.isfile(opath):
            self.logger.info("No LOG at %s; not complete." % opath)
            return False
        reachgrep=subprocess.Popen('grep "ERROR" %s' % opath, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        reachrpt=reachgrep.communicate()[0]
        reachgrep.wait()
        if reachrpt=='':
            founderror=False
        else:
            founderror=True
            raise MASTError(self.__class__.__name__,"Found error in LAMMPS Execution: {0}".format(reachrpt))
        reachgrep=subprocess.Popen('grep "CALCULATION HAS FINISHED" %s' % opath, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        reachrpt=reachgrep.communicate()[0]
        reachgrep.wait()
        if reachrpt=='':
            reachedend=False
        else:
            reachedend=True
        if reachedend:
            self.logger.info("LOG file at %s shows calculation has finished." % opath)
            return True
        else:
            self.logger.info("LOG file at %s does not show that calculation has finished; still incomplete." % opath)
            return False
    
    def is_ready_to_run(self):
        """Check if single LAMMPS non-NEB ingredient is 
            ready to run.
        """
        dirname = self.keywords['name']
        flist = os.listdir(dirname)
        notready = 0
        if "DATA" not in flist:
            notready += 1
        if "INPUT" not in flist:
            notready += 1
        if self.potential_file:
            potfile = os.path.basename(self.potential_file)
            if potfile not in flist:
                notready += 1
        if "submit.sh" not in flist:
            notready = notready + 1
        if notready > 0:
            return False
        else:
            exec_line = self.keywords['program_keys']['mast_exec']
            sp_exec_line = exec_line.split(' ')
            new_exec_line = str()
            for segment in sp_exec_line:
                if ('LOG' not in segment.upper()) and ('INPUT' not in segment):
                    new_exec_line += segment + ' '
            log_name = os.path.join(dirname,"LOG")
            in_name = os.path.join(dirname,"INPUT")
            new_exec_line += "-log {0} <{1}".format(log_name,in_name)
            self.keywords['program_keys']['mast_exec'] = new_exec_line
            return True
    
    def _lammps_data_setup(self):
        """Set up the DATA file for a single LAMMPS run.
        """
        name = self.keywords['name']
        datapath = os.path.join(name, "DATA")
        if os.path.isfile(datapath):
            my_data_structure = self.get_structure_from_file(datapath)
            #parent should have given a structure
        else: #this is an originating run; mast should give it a structure
            my_data_structure = self.keywords['structure']
            self.logger.info("No DATA file found from a parent; base structure used for %s" % self.keywords['name'])
        if 'mast_coordinates' in self.keywords['program_keys'].keys():
            sxtend = StructureExtensions(struc_work1=my_data_structure, name=self.keywords['name'])
            coordstrucs=self.get_coordinates_only_structure_from_input()
            newstruc = sxtend.graft_coordinates_onto_structure(coordstrucs[0])
            my_data_structure=newstruc.copy()
        #dirutil.lock_directory(name)
        my_data_atoms = AseAtomsAdaptor.get_atoms(my_data_structure)
        write_lammps_data(datapath, my_data_atoms)
        #dirutil.unlock_directory(name)
        return datapath
    
    def _lammps_input_get_non_mast_keywords(self):
        """Get the non-LAMMPS keywords and make a dictionary."""
        input_dict=dict()
        allowedpath = os.path.join(dirutil.get_mast_install_path(), 'MAST',
                        'ingredients','programkeys','lammps_allowed_keywords.py')
        allowed_list = self._lammps_input_get_allowed_keywords(allowedpath)
        for key, value in self.keywords['program_keys'].iteritems():
            if not key[0:5] == "mast_":
                keytry = key.upper()
                if not (keytry in allowed_list):
                    self.logger.warning("Ignoring program key %s for INPUT. To allow this keyword, add it to %s" % (keytry, allowedpath))
                else:
                    if type(value)==str and value.isalpha():
                        input_dict[keytry.lower()]=value
                    else:
                        input_dict[keytry.lower()]=value
        return input_dict

    def _lammps_input_get_allowed_keywords(self, allowedpath):
        """Get allowed vasp keywords.
            Args:
                allowedpath <str>: file path for allowed lammps keywords
        """
        allowed = MASTFile(allowedpath)
        allowed_list=list()
        for entry in allowed.data:
            allowed_list.append(entry.strip())
        return allowed_list
    
    def _lammps_input_setup(self, lammps_input_file="INPUT", lammps_data_file="DATA", lammps_trajectory_file="TRAJECTORY"):
        """Function to write a lammps input file
        Input:
            lammps_input_file = file to write lammps input to
            lammps_data_file = file to load lammps data from
            lammps_trajectory_file = file to write lammps trajectory to
        """
        if isinstance(lammps_input_file, str):
            f = open(lammps_input_file, 'w')
        else:
            f = lammps_input_file
        
        # Write file variable names
        f.write('clear\n' +
                ('variable dump_file string "%s"\n' % lammps_trajectory_file) +
                ('variable data_file string "%s"\n' % lammps_data_file))
        f.write('units metal \n')
        
        #Write to specified log file
        #logname = os.path.join(os.path.dirname(lammps_data_file),"LOG")
        #f.write('log %s append\n' % logname)
        
        #Get keyword parameters
        parameters = dict()
        parameters = self._lammps_input_get_non_mast_keywords()
        if 'autocorrect' in parameters:
            parameters['autocorrect'] = bool(parameters['autocorrect'])
        else:
            parameters['autocorrect'] = True
        if parameters['autocorrect']:
            try:
                atpath = os.path.join(os.path.dirname(lammps_data_file),'atom_symbols')
                try:
                    atf = open(atpath,'r')
                except:
                    atf = open('atom_symbols', 'r')
                atomlist = list()
                for line in atf.readlines():
                    atomlist.append(line.strip())
                atf.close()
                atomlist = sorted(list(set(atomlist)))
            except:
                struct = self.keywords['structure']
                atomst = AseAtomsAdaptor.get_atoms(struct)
                atomlist = sorted(list(set(atomst.get_chemical_symbols())))
            parameters = readable_lammps_parameters(parameters,atomlist)
        
        #Copy potential file to current working directory
        if ('potential_file' in parameters) and (parameters['potential_file'] != 'None'):
            newlocation = os.path.join(os.path.dirname(lammps_data_file),os.path.basename(parameters['potential_file']))
            shutil.copyfile(parameters['potential_file'],newlocation)
            self.potential_file = newlocation
        
        atoms = AseAtomsAdaptor.get_atoms(self.keywords['structure'])
        pbc = atoms.get_pbc()
        if ('boundary' in parameters) and (parameters['boundary'] != 'None'):
            f.write('boundary %s \n' % parameters['boundary'])
        else:
            f.write('boundary %c %c %c \n' % tuple('sp'[x] for x in pbc))
        f.write('atom_modify sort 0 0.0 \n')
        if ('neighbor' in parameters) and (parameters['neighbor'] != 'None'):
            f.write('neighbor %s \n' % parameters['neighbor'])
        if ('newton' in parameters) and (parameters['newton'] != 'None'):
            f.write('newton %s \n' % parameters['newton'])
                
        f.write('\n')
        
        f.write('read_data %s\n' % lammps_data_file)
        
        # Write pair potential information
        f.write('\n### interactions \n')
        if ( ('pair_style' in parameters) and ('pair_coeff' in parameters) ):
            if parameters['pair_style'] == 'None':
                raise MASTError(self.__class__.__name__,"No pair_style provided for LAMMPS Input file")
            pair_style = parameters['pair_style']
            f.write('pair_style %s \n' % pair_style)
            if ('None' in parameters['pair_coeff']):
                raise MASTError(self.__class__.__name__,"No pair_coeff provided for LAMMPS Input file")
            if not isinstance(parameters['pair_coeff'],list):
                try:
                    parameters['pair_coeff'] = eval(parameters['pair_coeff'])
                except:
                    parameters['pair_coeff'] = [ parameters['pair_coeff'] ]
            for pair_coeff in parameters['pair_coeff']:
                f.write('pair_coeff %s \n' % pair_coeff)
            if ('mass' in parameters) and (parameters['mass'] != 'None'):
                if not isinstance(parameters['mass'],list):
                    try:
                        parameters['mass'] = eval(parameters['mass'])
                    except:
                        parameters['mass'] = [ parameters['mass'] ]
                for mass in parameters['mass']:
                    f.write('mass %s \n' % mass)
        else:# Use simple leonnard jones calculation
            f.write('pair_style lj/cut 2.5 \n' +
                    'pair_coeff * * 1 1 \n' +
                    'mass * 1.0 \n')
        #Set up min_style if appropriate
        if ('min_style' in parameters) and (parameters['min_style'] != 'None'):
            f.write('min_style %s \n' % parameters['min_style'])
        if ('min_modify' in parameters) and (parameters['min_modify'] != 'None'):
            f.write('min_modify %s \n' % parameters['min_modify'])
        
        #Write line for thermo arguments and stemps
        thermo_args = ['step', 'temp', 'press', 'cpu', 
                            'pxx', 'pyy', 'pzz', 'pxy', 'pxz', 'pyz',
                            'ke', 'pe', 'etotal',
                            'vol', 'lx', 'ly', 'lz', 'atoms']
        thermo_mark = ' '.join([x.capitalize() for x in thermo_args[0:3]])
        if ('thermosteps' not in parameters) or (parameters['thermosteps']=='None'):
            parameters['thermosteps'] = '1'
        
        f.write('\n### run\n' + 
                 'fix fix_nve all nve\n'+
                ('dump dump_all all custom {0} {1}'.format(parameters['thermosteps'], lammps_trajectory_file))+
                ' id type x y z vx vy vz fx fy fz'+
                '\n')
        f.write(('thermo_style custom %s\n' +
                'thermo_modify flush yes\n' +
                'thermo '+repr(parameters['thermosteps'])+
                '\n') % (' '.join(thermo_args)))

        rflag = False
        if ('minimize' in parameters) and (parameters['minimize'] != 'None'):
            f.write('minimize %s\n' % parameters['minimize'])
            rflag = True
        if ('run' in parameters) and (parameters['run'] != 'None'):
            f.write('run %s\n' % parameters['run'])
            rflag = True
        if not rflag:
            f.write('run 0\n')
        
        f.write('print "CALCULATION HAS FINISHED"\n')
        f.write('log /dev/stdout\n') # Force LAMMPS to flush log
        f.write('undump dump_all\n') # Force LAMMPS to flush trj
        f.close()
        return lammps_input_file
    
    def set_up_program_input(self):
        """Set up the program input files."""
        datapath = self._lammps_data_setup()
        in_name = os.path.join(os.path.dirname(datapath),"INPUT")
        traj_name = os.path.join(os.path.dirname(datapath),"TRAJECTORY")
        log_name = os.path.join(os.path.dirname(datapath),"LOG")
        self._lammps_input_setup(lammps_input_file = in_name, lammps_data_file = datapath, lammps_trajectory_file=traj_name)
        exec_line = self.keywords['program_keys']['mast_exec']
        sp_exec_line = exec_line.split(' ')
        new_exec_line = str()
        for segment in sp_exec_line:
            if ('LOG' not in segment.upper()) and ('INPUT' not in segment):
                new_exec_line += segment + ' '
        new_exec_line += "-log {0} <{1}".format(log_name,in_name)
        self.keywords['program_keys']['mast_exec'] = new_exec_line
        return

    def get_energy(self):
        """Get the energy.
        """
        thermo_content = read_lammps_log()
        return thermo_content[-1]['pe']
    
    def get_energy_from_energy_file(self):
        """Get the energy from the energy file.
            Args:
                mydir <str>: Directory in which to look.
            Returns:
                <float>: last energy from LOG file
        """
        fullpath=os.path.join(self.keywords['name'], "LOG")
        if not os.path.isfile(fullpath):
            raise MASTError(self.__class__.__name__, "No LOG file at %s" % self.keywords['name'])
        thermo_content = read_lammps_log(fullpath)
        if len(thermo_content) ==0:
            self.logger.error("Failed to log energy %s" % str(fullpath))
            return 1000
        else:
            return thermo_content[-1]['pe']

    def is_started(self):
        """See if the ingredient has been started on
            the queue.
        """
        if os.path.isfile(os.path.join(self.keywords['name'],'TRAJECTORY')):
            return True
        else:
            return False

    def write_final_structure_file(self, mystruc):
        """Write the final structure to a file.
            For LAMMPS, this is Trajectory.
        """
        write_lammps_trajectory(os.path.join(self.keywords['name'],'TRAJECTORY'),mystruc)

    def has_starting_structure_file(self):
        """Evaluate whether the ingredient has a starting
            structure file. For LAMMPS, this is a DATA file.
        """
        return os.path.isfile(os.path.join(self.keywords['name'], 'DATA'))

    def has_ending_structure_file(self):
        """Evaluate whether the ingredient has a starting
            structure file. For LAMMPS, this is a TRAJECTORY.
        """
        return os.path.isfile(os.path.join(self.keywords['name'], 'TRAJECTORY'))

    def get_final_pressure(self):
        """Get the current calculated pressure, assume isotropic medium.
            in Bar
        """
        fullpath=os.path.join(self.keywords['name'], "LOG")
        if not os.path.isfile(fullpath):
            raise MASTError(self.__class__.__name__, "No LOG file at %s" % self.keywords['name'])
        thermo_content = read_lammps_log(fullpath)
        tc = thermo_content[-1]
        stress = np.array([tc[i] for i in ('pxx','pyy','pzz')])
        return (-(stress[0] + stress[1] + stress[2]) / 3.0)

def write_lammps_data(data_name, atoms):
    """Function to write ase atoms object to LAMMPS file
    Input:
        data_name = String or fileobject for data to be written to
        atoms = ase atoms object to be written
    Output:
        None. Data file will be written
    """
    if isinstance(data_name, str):
        f = open(data_name, 'w')
    else:
        f = data_name
    f.write('\n\n')
    symbols = atoms.get_chemical_symbols()
    n_atoms = len(symbols)
    f.write('{0} \t atoms \n'.format(n_atoms))
    #Order species alphabetically
    ordered_symbols = sorted(list(set(symbols)))
    #Write these symbols to a file to ensure they are kept
    dir = os.path.dirname(data_name)
    afilename = os.path.join(dir, 'atom_symbols')
    a = open(afilename,'w')
    for sym in ordered_symbols:
        a.write('{0}\n'.format(sym))
    a.close()
    n_types = len(ordered_symbols)
    f.write('{0}  atom types\n'.format(n_types))
    p = prism(atoms.get_cell())
    xhi, yhi, zhi, xy, xz, yz = p.get_lammps_prism_str()

    f.write('0.0 %s  xlo xhi\n' % xhi)
    f.write('0.0 %s  ylo yhi\n' % yhi)
    f.write('0.0 %s  zlo zhi\n' % zhi)

    if p.is_skewed():
        f.write('%s %s %s  xy xz yz\n' % (xy, xz, yz))
    f.write('\n\n')

    f.write('Atoms \n\n')
    for i, r in enumerate(map(p.pos_to_lammps_str,
                              atoms.get_positions())):
        s = ordered_symbols.index(symbols[i]) + 1
        f.write('%6d %3d %s %s %s\n' % ((i+1, s)+tuple(r)))
    f.close()

def read_lammps_trajectory(filename, timestep=-1, atomlist=False,
    ratomlist=False, writefile=False):
    """Function to convert LAMMPS Trajectory file to ASE atoms object
    Inputs:
        filename = string of LAMMPS Trajectory file to read
        timestep = Which timestep to read from the trajectory file.
            Default is last time step available
            If timestep is set to all then outputs become lists
        atomlist = list of element strings corresponding to number type
        ratomlist = Boolean for whether or not to return the atom list
        write_file = Boolean for whether or not to write ASE Atoms object to file
    Ouput:
        ASE atoms object containing structure from timestep specified
        if ratomlist: list containing element strings applied to structure corresponding to LAMMPS number
        vlist = list of velocities for each atom in structure at timestep
        flist = list of forcer for each atom in structure at timestep
    """
    f = open(filename,'r')
    if atomlist==False:
        atnum = []
        attype = []
    alist = []
    vlist = []
    flist = []
    timeflag = False
    numberflag = False
    boxflag = False
    atomsflag = False
    for line in f.readlines():
        sp = line.split()
        if timeflag == True:
            timeflag = False
        elif numberflag == True:
            noa = float(sp[0])
            numberflag = False
        elif boxflag == True:
            if boxcount <= 2:
                box.append(float(sp[1]))
                boxcount += 1
            if boxcount > 2:
                boxflag = False
        elif atomsflag == True:
            if na < noa:
                if atomlist:
                    sym = atomlist[[i for i,sym in enumerate(atomlist) if str(i+1)==sp[1]][0]]
                    at = Atom(symbol=sym, position=[float(sp[2]),float(sp[3]),float(sp[4])])
                    a.append(at)
                    v = [float(sp[i]) for i in range(5,8)]
                    vs.append(v)
                    f1 = [float(sp[i]) for i in range(8,11)]
                    fs.append(f1)
                    na += 1
                else:
                    if sp[1] in atnum:
                        sym = attype[[i for i,num in enumerate(atnum) if num==sp[1]][0]]
                        at=Atom(symbol=sym,position=[float(sp[2]),float(sp[3]),float(sp[4])])
                        a.append(at)
                    else:
                        symn = random.choice(range(1,100))
                        at=Atom(symbol=symn,position=[float(sp[2]),float(sp[3]),float(sp[4])])
                        a.append(at)
                        atnum.append(sp[1])
                        attype.append(at.symbol)
                    v = [float(sp[i]) for i in range(5,8)]
                    vs.append(v)
                    f1 = [float(sp[i]) for i in range(8,11)]
                    fs.append(f1)
                    na += 1
            if na >= noa:
                atomsflag = False
                if atomlist==False:
                    atomlist=list(np.zeros(len(attype)))
                    for i in range(len(atnum)):
                        atomlist[int(atnum[i])-1]=attype[i]
                alist.append(a)
                vlist.append(vs)
                flist.append(fs)
        elif sp[1] == 'TIMESTEP':
            timeflag = True
        elif sp[1] == 'NUMBER':
            numberflag = True
        elif sp[1] == 'BOX':
            boxflag = True
            box = []
            boxcount = 0
            pbcc = [False,False,False]
            if sp[3] == 'pp':
                pbcc[0] = True
            if sp[4] == 'pp':
                pbcc[1] = True
            if sp[5] == 'pp':
                pbcc[2] = True
        elif sp[1] == 'ATOMS':
            atomsflag = True
            na = 0
            a = Atoms(cell=box,pbc=pbcc)
            vs = []
            fs = []
    f.close()
    if timestep == 'all':
        writefile=False
    if writefile == True:
        write_xyz(filename+'.xyz',alist[timestep],'xyz')
    if timestep == 'all':
        if ratomlist:
            return alist, atomlist, vlist, flist
        else:
            return alist, vlist, flist
    else:
        if ratomlist:
            return alist[timestep], atomlist, vlist[timestep], flist[timestep]
        else:
            return alist[timestep], vlist[timestep], flist[timestep]

def write_lammps_trajectory(filename, structure):
    if isinstance(filename,str):
        f = open(filename,'w')
    else:
        f = filename
    f.write("ITEM: TIMESTEP\n0\nITEM: NUMBER OF ATOMS\n")
    atoms = AseAtomsAdaptor.get_atoms(structure)
    f.write("{0}\n".format(len(atoms)))
    f.write("ITEM: BOX BOUNDS ")
    pbc = atoms.get_pbc()
    for bound in pbc:
        if bound:
            f.write("pp ")
        else:
            f.write("s ")
    f.write("\n")
    p = prism(atoms.get_cell())
    xhi, yhi, zhi, xy, xz, yz = p.get_lammps_prism_str()

    f.write('0.0 {0}\n'.format(xhi))
    f.write('0.0 {0}\n'.format(yhi))
    f.write('0.0 {0}\n'.format(zhi))
    if p.is_skewed():
        f.write('%s %s %s\n' % (xy, xz, yz))
    f.write('ITEM: ATOMS id type x y z vx vy vz fx fy fz\n')
    syms = sorted(list(set(atoms.get_chemical_symbols())))
    symlist = [one for one in enumerate(syms)]
    for idx in range(len(atoms)):
        type = [ty+1 for ty,sym in symlist if sym==atoms[idx].symbol][0]
        f.write('{0} {1} {2}'.format(idx+1,type, ' '.join([repr(x) for x in atoms[0].position])))
        f.write('0 0 0 0 0 0\n')
    f.close()
    return

def read_lammps_data(filename, atomlist=False, ratomlist=False, writefile=False):
    """Function to read a Lammps data file and output an xyz and atoms object
    Input:
        Filename = string for data file to read
        atomlist = List of strings corresponding to Lammps atom type
            ['Cr','Fe','He',...]
        ratomlist = True/False - will return atomlist with structure
        writefile = True/False - will output an xyz file of the Lammps data file
    """
    f=open(filename,'r')
    for i in range(4):
        ln=f.readline()
    box = []
    for i in range(3):
        ln=f.readline().split()
        box.append(float(ln[1]))
    for i in range(4):
        ln=f.readline()
    a=Atoms(cell=box)
    if atomlist:
        atlist=[pair for pair in enumerate(atomlist)]
        for line in f.readlines():
            sp=line.split()
            for i,sym in atlist:
                if sp[1]==str(i+1):
                    at=Atom(symbol=sym,position=[float(sp[2]),float(sp[3]),float(sp[4])])
                    a.append(at)
    else:
        atnum = []
        attype = []
        for line in f.readlines():
            sp=line.split()
            if sp[1] in atnum:
                sym = attype[[i for i,num in enumerate(atnum) if num==sp[1]][0]]
                at=Atom(symbol=sym,position=[float(sp[2]),float(sp[3]),float(sp[4])])
                a.append(at)
            else:
                symn = random.choice(range(1,100))
                at=Atom(symbol=symn,position=[float(sp[2]),float(sp[3]),float(sp[4])])
                a.append(at)
                atnum.append(sp[1])
                attype.append(at.symbol)
        atomlist=list(np.zeros(len(attype)))
        for i in range(len(atnum)):
            atomlist[int(atnum[i])-1]=attype[i]
    f.close()
    if writefile==True:
        write_xyz(filename+'.xyz',a,'xyz')
    if ratomlist:
        return a, atomlist
    else:
        return a

def read_lammps_log(lammps_log="LOG"):
    """Method which reads a LAMMPS output log file."""

    if isinstance(lammps_log, str):
        f = open(lammps_log, 'r')
    else:
        f = lammps_log
    #thermo arguments
    thermo_args = ['step', 'temp', 'press', 'cpu', 
                        'pxx', 'pyy', 'pzz', 'pxy', 'pxz', 'pyz',
                        'ke', 'pe', 'etotal',
                        'vol', 'lx', 'ly', 'lz', 'atoms']
    thermo_mark = ' '.join([x.capitalize() for x in thermo_args[0:3]])
    f_re = r'([+-]?(?:(?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?|nan|inf))'
    n = len(thermo_args)
    thermo_re = re_compile(r'^\s*' + r'\s+'.join([f_re]*n) + r'\s*$', flags=IGNORECASE)
    thermo_content = []
    line = f.readline()
    while line and "CALCULATION HAS FINISHED" not in line.strip():
        if line.startswith(thermo_mark):
            m = True
            while m:
                line = f.readline()
                m = thermo_re.match(line)
                if m:
                    # create a dictionary between each of the thermo_style args
                    # and it's corresponding value
                    thermo_content.append(dict(zip(thermo_args, map(float, m.groups()))))
        else:
            line = f.readline()
    f.close()
    return thermo_content

def readable_lammps_parameters(parameters, atomlist):
    #atoms = AseAtomsAdaptor.get_atoms(structure)
    #atomlist = sorted(list(set(atoms.get_chemical_symbols())))
    masslist = [Atom(sym).mass for sym in atomlist]
    if 'potential_file' in parameters:
        if parameters['potential_file']:
            parameters['pot_file'] = os.path.basename(parameters['potential_file'])
    if parameters['pair_style']=='tersoff':
        parcoff = '* * {0}'.format(parameters['pot_file'])
        for one in atomlist:
            parcoff+=' {0}'.format(one[0])
        parameters['pair_coeff'] = [parcoff]
        mass = ['1 {0}'.format(masslist[0])]
        if len(masslist) > 1:
            for i in range(len(masslist)-1):
                mass.append('{0} {1}'.format(i+2,masslist[i+1]))
        parameters['mass'] = mass
    elif parameters['pair_style']=='eam':
        pair_coeff = [ '* * {0}'.format(parameters['pot_file'])]
        parameters['pair_coeff'] = pair_coeff
    elif parameters['pair_style']=='eam/fs':
        parcoff = '* * {0}'.format(parameters['pot_file'])
        for one in atomlist:
            parcoff+=' {0}'.format(one[0])
        parameters['pair_coeff'] = [parcoff]
        mass = ['1 {0}'.format(masslist[0])]
        if len(masslist) > 1:
            for i in range(len(masslist)-1):
                mass.append('{0} {1}'.format(i+2,masslist[i+1]))
    elif parameters['pair_style']=='eam/cd':
        parcoff = '* * {0}'.format(parameters['pot_file'])
        for one in atomlist:
            parcoff+=' {0}'.format(one[0])
        parameters['pair_coeff'] = [parcoff]
    elif parameters['pair_style']=='edip':
        parcoff = '* * {0}'.format(parameters['pot_file'])
        for one in atomlist:
            parcoff+=' {0}'.format(one)
        parameters['pair_coeff'] = [parcoff]
        mass = ['1 {0}'.format(masslist[0])]
        if len(masslist) > 1:
            for i in range(len(masslist)-1):
                mass.append('{0} {1}'.format(i+2,masslist[i+1]))
        parameters['mass'] = mass
        parameters['newton'] = 'on'
    elif parameters['pair_style']=='bop':
        parcoff = '* * {0}'.format(parameters['pot_file'])
        for one in atomlist:
            parcoff+=' {0}'.format(one[0])
        parcoff+='\ncommunicate single cutoff {0}'.format(parameters['bopcutoff'])
        parameters['pair_coeff'] = [parcoff]
        mass = ['1 {0}'.format(masslist[0])]
        if len(masslist) > 1:
            for i in range(len(masslist)-1):
                mass.append('{0} {1}'.format(i+2,masslist[i+1]))
        parameters['mass'] = mass
        parameters['newton'] = 'on'
    elif parameters['pair_style']=='buck':
        pairstyle='{0} {1}'.format(parametes['pair_style'], parameters['buckcutoff'])
        pair_coeff=parameters['buckparameters']
        mass = ['1 {0}'.format(masslist[0])]
        if len(masslist) > 1:
            for i in range(len(masslist)-1):
                mass.append('{0} {1}'.format(i+2, masslist[i+1]))	
        parameters['pair_style'] = pairstyle
        parameters['pair_coeff'] = pair_coeff
        parameters['mass'] = mass
        parameters['potential_file'] = 'None'
    elif parameters['pair_style']=='other':
        mass = ['1 {0}'.format(masslist[0])]
        if len(masslist) > 1:
            for i in range(len(masslist)-1):
                mass.append('{0} {1}'.format(i+2,masslist[i+1]))
        if 'charges' in parameters:
            cs = parameters['charges']
            parameters['mass'][len(parameters['mass'])-1] +=cs[1]
            parameters['newton']+='\natom_style charge'
    else:
        parameters={'potential_file':'None'}
        print 'WARNING: No LAMMPS potential recognized. Assuming Lennard Jones Potential'
    return parameters

class prism:
    def __init__(self, cell, pbc=(True,True,True), digits=10):
        """COPYRIGHT: ATOMISTIC SIMULATION ENVIRONMENT
        PLEASE SEE https://wiki.fysik.dtu.dk/ase/ FOR SIMILAR METHODS
        AND MORE INFORMATION ON THIS CLASS STRUCTURE
        
        Create a lammps-style triclinic prism object from a cell

        The main purpose of the prism-object is to create suitable 
        string representations of prism limits and atom positions
        within the prism.
        When creating the object, the digits parameter (default set to 10)
        specify the precission to use.
        lammps is picky about stuff being within semi-open intervals,
        e.g. for atom positions (when using create_atom in the in-file), 
        x must be within [xlo, xhi).
        """
        a, b, c = cell
        an, bn, cn = [np.linalg.norm(v) for v in cell]
        
        alpha = np.arccos(np.dot(b, c)/(bn*cn))
        beta  = np.arccos(np.dot(a, c)/(an*cn))
        gamma = np.arccos(np.dot(a, b)/(an*bn))
        
        xhi = an
        xyp = np.cos(gamma)*bn
        yhi = np.sin(gamma)*bn
        xzp = np.cos(beta)*cn
        yzp = (bn*cn*np.cos(alpha) - xyp*xzp)/yhi
        zhi = np.sqrt(cn**2 - xzp**2 - yzp**2)
    
        # Set precision
        self.car_prec = dec.Decimal('10.0') ** \
            int(np.floor(np.log10(max((xhi,yhi,zhi))))-digits)
        self.dir_prec = dec.Decimal('10.0') ** (-digits)
        self.acc = float(self.car_prec)
        self.eps = np.finfo(xhi).eps

        # For rotating positions from ase to lammps
        Apre = np.array(((xhi, 0,   0),
                         (xyp, yhi, 0),
                         (xzp, yzp, zhi)))
        self.R = np.dot(np.linalg.inv(cell), Apre)

        # Actual lammps cell may be different from what is used to create R
        def fold(vec, pvec, i):
            p = pvec[i]
            x = vec[i] + 0.5*p
            n = (np.mod(x, p) - x)/p
            return [float(self.f2qdec(a)) for a in (vec + n*pvec)]

        Apre[1,:] = fold(Apre[1,:], Apre[0,:], 0)
        Apre[2,:] = fold(Apre[2,:], Apre[1,:], 1)
        Apre[2,:] = fold(Apre[2,:], Apre[0,:], 0)

        self.A = Apre
        self.Ainv = np.linalg.inv(self.A)

        if self.is_skewed() and \
                (not (pbc[0] and pbc[1] and pbc[2])):
            raise RuntimeError('Skewed lammps cells MUST have '
                               'PBC == True in all directions!')

    def f2qdec(self, f):
        return dec.Decimal(repr(f)).quantize(self.car_prec, dec.ROUND_DOWN)

    def f2qs(self, f):
        return str(self.f2qdec(f))

    def f2s(self, f):
        return str(dec.Decimal(repr(f)).quantize(self.car_prec, dec.ROUND_HALF_EVEN))

    def dir2car(self, v):
        "Direct to cartesian coordinates"
        return np.dot(v, self.A)

    def car2dir(self, v):
        "Cartesian to direct coordinates"
        return np.dot(v, self.Ainv)

    def fold_to_str(self,v):
        "Fold a position into the lammps cell (semi open), return a tuple of str" 
        # Two-stage fold, first into box, then into semi-open interval
        # (within the given precission).
        d = [x % (1-self.dir_prec) for x in 
             map(dec.Decimal, map(repr, np.mod(self.car2dir(v) + self.eps, 1.0)))]
        return tuple([self.f2qs(x) for x in 
                      self.dir2car(map(float, d))])
        
    def get_lammps_prism(self):
        A = self.A
        return (A[0,0], A[1,1], A[2,2], A[1,0], A[2,0], A[2,1])

    def get_lammps_prism_str(self):
        "Return a tuple of strings"
        p = self.get_lammps_prism()
        return tuple([self.f2s(x) for x in p])

    def pos_to_lammps_str(self, position):
        "Rotate an ase-cell postion to the lammps cell orientation, return tuple of strs"
        return tuple([self.f2s(x) for x in np.dot(position, self.R)])

    def pos_to_lammps_fold_str(self, position):
        "Rotate and fold an ase-cell postion into the lammps cell, return tuple of strs"
        return self.fold_to_str(np.dot(position, self.R))

    def is_skewed(self):
        acc = self.acc
        prism = self.get_lammps_prism()
        axy, axz, ayz = [np.abs(x) for x in prism[3:]]
        return (axy >= acc) or (axz >= acc) or (ayz >= acc)
        



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import os
import time
import shutil
import logging

from MAST.utility import MASTObj
from MAST.utility import MAST2Structure
from MAST.utility import MASTError
from MAST.utility import MASTFile
#from MAST.utility.picklemanager import PickleManager
from MAST.utility.dirutil import *
from MAST.utility import InputOptions
from MAST.utility import loggerutils
from MAST.parsers import InputParser
from MAST.parsers import IndepLoopInputParser
#from MAST.parsers import InputPythonCreator
from MAST.parsers.recipetemplateparser import RecipeTemplateParser
from MAST.recipe.recipesetup import RecipeSetup

from MAST.ingredients import *

ALLOWED_KEYS     = {\
                       'inputfile'    : (str, 'mast.inp', 'Input file name'),\
                       'outputfile'   : (str, 'mast.out', 'Output file name'),\
                   } 


class MAST(MASTObj):
    """User interface to set up a calculation group or a set of
        independently and/or pegged looped calculation groups.

        Attributes:
            self.input_options <InputOptions object>: Stores the
                options parsed from the input file
            self.recipe_plan <RecipePlan object>: Stores the
                recipe plan parsed from self.input_options and
                the personalized recipe file
            self.origin_dir <str>: Original directory for the
                                   mast -i *.inp command
            self.timestamp <str>: Timestamp of the mast -i *.inp
                                  command
            self.asctime <str>: ASCII timestamp
            self.working_directory <str>: Working directory 
                created for new recipe from mast -i *.inp command
            self.sysname <str>: System name (elements)
            self.logger <logging logger>
    """
    def __init__(self, **kwargs):
        MASTObj.__init__(self, ALLOWED_KEYS, **kwargs)
        self.input_options = None
        self.origin_dir=""
        self.timestamp=""
        self.asctime=""
        self.working_directory=""
        self.sysname=""
        self.recipe_plan = None
        self.logger = loggerutils.initialize_short_logger(os.path.join(os.getenv("MAST_CONTROL"),"mast.log"))


    def check_independent_loops(self):
        """Checks for independent loops. If no independent loops are found,
            parse and run the input file. Otherwise, parse and run the
            generated set of input files.
        """
        self.logger.info("%%%%%%%%%%%%%%%%%%%%%%%%%%%%")
        self.logger.info("\nMAST input started at %s with inputfile %s.\n" % (time.asctime(),self.keywords['inputfile']))
        self.logger.info("%%%%%%%%%%%%%%%%%%%%%%%%%%%%")
        try:
            ipl_obj = IndepLoopInputParser(inputfile=self.keywords['inputfile'])
            loopfiles = ipl_obj.main()
            #print "TTM DEBUG: loopfiles: ", loopfiles
            if len(loopfiles) == 0:
                self.set_up_recipe()
            else:
                for ipfile in loopfiles:
                    self.keywords['inputfile']=ipfile
                    self.set_up_recipe()
        except MASTError as errormsg:
            self.logger.error(str(errormsg))
        self.logger.info("%%%%%%Finished processing inputfile.%%%%%%")


    def set_input_options(self):
        """Set input options.
        """
        parser_obj = InputParser(inputfile=self.keywords['inputfile'])
        self.input_options = parser_obj.parse()


    def set_up_recipe(self):
        """Set up the recipe.
            Set class attributes, 
            create the recipe directory,
            write recipe directory-level info,
            create the recipe plan (which creates ingredient
            directories), 
            and archive the recipe plan and input options.
        """
        self.set_input_options()
        self.set_class_attributes()
        self.make_working_directory()
        self.create_recipe_metadata()
        self.copy_input_file()
        self.parse_recipe_template()
        self.create_recipe_plan()
        self.create_archive_files()

    def create_recipe_plan(self):
        """Create the recipe plan object, and print its status.
        """
        setup_obj = RecipeSetup(recipeFile=os.path.join(self.working_directory,'personal_recipe.txt'), 
                inputOptions=self.input_options,
                structure=self.input_options.get_item('structure','structure'), 
                workingDirectory=self.working_directory
                )
        self.recipe_plan = setup_obj.start()
        self.recipe_plan.print_status()
        return

    def make_working_directory(self):
        """Initialize the directory for writing the 
            recipe and ingredient folders.
        """
        try:
            os.mkdir(self.working_directory)
        except:
            MASTError(self.__class__.__name__, "Cannot create working directory %s !!!" % self.working_directory)

    def copy_input_file(self):
        """Copy the input file to input.inp
            If the original file had loops, this is a single,
            non-looped input file.
        """
        shutil.copy(self.keywords['inputfile'], 
            os.path.join(self.working_directory,'input.inp'))

    def create_recipe_metadata(self):
        """Create the recipe metadata file.
        """
        topmeta = Metadata(metafile="%s/metadata.txt" % self.working_directory)
        topmeta.write_data('directory_created', self.asctime)
        topmeta.write_data('system_name', self.sysname)
        topmeta.write_data('origin_dir', self.origin_dir)
        topmeta.write_data('working_directory', self.working_directory)
        topmeta.write_data('timestamp', self.timestamp)
        return

    def create_archive_files(self):
        """Save off archive files.
            Returns:
                creates archive_input_options.txt
                creates archive_recipe_plan.txt
        """
        inputsave = MASTFile()
        inputsave.data = repr(self.input_options)
        inputsave.to_file(os.path.join(self.working_directory, 'archive_input_options.txt'))

        recipesave = MASTFile()
        recipesave.data = repr(self.recipe_plan)
        recipesave.to_file(os.path.join(self.working_directory, 'archive_recipe_plan.txt'))

        #pickle_plan = os.path.join(self.working_directory, 'archive_recipe_plan.pickle')
        #pm = PickleManager(pickle_plan)
        #pm.save_variable(self.recipe_plan)
        
        #pickle_options = os.path.join(self.working_directory, 'archive_input_options.pickle')
        #pm = PickleManager(pickle_options)
        #pm.save_variable(self.input_options)

        #create the *.py input script
        #ipc_obj = InputPythonCreator(input_options=self.input_options)
        #ipc_filename = ipc_obj.write_script(self.working_directory, 'archive_input_options.py')
        return

    def parse_recipe_template(self):
        """Parses the recipe template file."""

        recipe_file = self.input_options.get_item('recipe', 'recipe_file')

        parser_obj = RecipeTemplateParser(templateFile=recipe_file, 
            inputOptions=self.input_options,
            personalRecipe=os.path.join(self.working_directory,'personal_recipe.txt'),
            working_directory=self.working_directory
            )
        parser_obj.parse()

    def set_class_attributes(self):
        """Set class attributes, other than input options
        """
        time.sleep(1)
        self.timestamp = time.strftime('%Y%m%dT%H%M%S')
        self.asctime = time.asctime()
        self.set_sysname()
        self.set_origin_dir()
        self.set_working_directory()
    
    def set_origin_dir(self):
        """Set the origin directory (input file directory)
        """
        self.origin_dir = os.path.dirname(self.keywords['inputfile'])
        if self.origin_dir == "":
            self.origin_dir = os.getcwd()
        return

    def set_sysname(self):
        """Set system name."""
        element_str = self.get_element_map_string()
        if element_str == None:
            element_str = self.get_element_string()
        system_name = self.input_options.get_item("mast", "system_name", "sys")
        self.sysname = system_name + '_' + element_str
        return

    def set_working_directory(self):
        """Get the system name and working directory.
        """
        recipename = os.path.basename(self.input_options.get_item('recipe','recipe_file')).split('.')[0]
        #dir_name = "%s_%s_%s" % (self.sysname, recipename, self.timestamp)
        dir_name = "%s_%s" % (self.sysname, self.timestamp)
        dir_path = str(os.path.join(os.getenv("MAST_SCRATCH"), dir_name))
        self.working_directory = dir_path
        return

    def get_element_string(self):
        """Get the element string from the structure.
        """
        mystruc = self.input_options.get_item('structure','structure')
        elemset = set(mystruc.species)
        elemstr=""
        elemok=1
        while elemok == 1:
            try:
                elempop = elemset.pop()
                elemstr = elemstr + elempop.symbol
            except KeyError:
                elemok = 0
        return elemstr
    def get_element_map_string(self):
        """Get the element string from the elementmap section.
            Returns:
                elemstr <str>: element string from elementmap
                                section of the input file,
                                in order of X1, X2, etc.
                If no elementmap section exists in the input file
                (which, when put in dictionary form, has the
                subsection name element_map), then the method
                returns None
        """
        elemmap = self.input_options.get_item('structure','element_map')
        if elemmap == None:
            return None
        elkeys = elemmap.keys()
        elkeys.sort()
        elstr=""
        for elkey in elkeys:
            elstr = elstr + elemmap[elkey]
        if elstr == "":
            return None
        return elstr





import random

def fuss2(pop, nkeep, Optimizer):
    """Selection function to employ basic Fixed Uniform Selection Scheme to select and 
	order individuals in a population.
	fuss point must reside within fusslimit of minimum fitness
	"""
    newpop = []
    fits = [ind.fitness for ind in pop]
    minindex = min(xrange(len(fits)), key=fits.__getitem__)
    minf = min(fits)
    maxf = max(fits)
    if abs(maxf - minf) > fusslimit:
        maxf = minf + fusslimit
    indices = []
    loopcount = 0
    while len(newpop) < nkeep:
        pt = random.uniform(minf,maxf)
        i = 0
        distances = []
        for fit in fits:
            distances.append((abs(pt - fit),i))
            i += 1
        dlist = sorted(distances, key=lambda one: one[0])
        index = dlist[0][1]
        if index not in indices:
            newpop.append(pop[index])
            indices.append(index)
        else:
            loopcount += 1
        if loopcount > 100:
            indexl = [l for l in range(len(pop)) if l not in indices]
            if len(indexl)==0:
                randi = random.choice(range(len(pop)))
                newpop.append(pop[randi])
                indices.append(randi)
            else:
                newpop.append(pop[min(indexl)])
                indices.append(min(indexl))
    if minindex not in indices:
        rm = random.choice(indices)
        newpop = [pop[inx] for inx in indices if inx != rm]
        newpop.append(pop[minindex])
    return newpop